import os

from neuromllite.utils import load_simulation_json
from pyneuroml.pynml import get_value_in_si

from pyneuroml import pynml
from pyelectro import analysis
from collections import OrderedDict

from neuromllite.utils import print_v

from matplotlib import pyplot as plt
import numpy as np

import sys
import random
import time

"""


        Work in progress!!!
        
        
"""

"""
    Can be used to run multiple simulations across a range of parameters
"""


class ParameterSweep:
    def __init__(
        self,
        runner,
        vary,
        fixed={},
        verbose=False,
        num_parallel_runs=1,
        plot_all=False,
        save_plot_all_to=None,
        heatmap_all=False,
        save_heatmap_to=None,
        heatmap_lims=None,
        show_plot_already=False,
        peak_threshold=0,
    ):

        self.sim = load_simulation_json(runner.nmllite_sim)

        self.colormap = "jet"

        ps_id = "ParamSweep_%s_%s" % (
            self.sim.id,
            time.ctime().replace(" ", "_").replace(":", "."),
        )

        print_v(
            "Initialising ParameterSweep %s with %s, %s (%i parallel runs)"
            % (ps_id, vary, fixed, num_parallel_runs)
        )

        os.mkdir(ps_id)
        self.result_dir = os.path.abspath(ps_id)

        self.runner = runner
        self.fixed = fixed
        self.vary = vary
        self.verbose = verbose
        self.num_parallel_runs = num_parallel_runs

        self.plot_all = plot_all
        self.save_plot_all_to = save_plot_all_to
        self.heatmap_all = heatmap_all
        self.save_heatmap_to = save_heatmap_to
        self.heatmap_lims = heatmap_lims
        self.show_plot_already = show_plot_already

        self.index = 0
        self.total_todo = 1
        self.report = OrderedDict()
        self.report["Varied parameters"] = vary
        self.report["Fixed parameters"] = fixed
        self.report["Simulations"] = OrderedDict()
        for v in vary:
            self.total_todo *= len(vary[v])

        self.analysis_var = {
            "peak_delta": 0,
            "baseline": 0,
            "dvdt_threshold": 0,
            "peak_threshold": peak_threshold,
        }

        if self.plot_all:
            self.ax = pynml.generate_plot(
                [],
                [],  # Add 2 sets of y values
                "Traces generated from %s" % self.sim.id,  # Title
                labels=[],
                xaxis="Time (ms)",  # x axis legend
                yaxis="Membrane potential (mV)",  # y axis legend
                title_above_plot=True,
                show_plot_already=False,
            )  # Save figure

        if self.heatmap_all:
            if len(self.vary) != 1:
                raise Exception(
                    "Heatmap can only be used when only one parameter is varied..."
                )
            self.hm_x = None
            self.hm_y = []
            self.hm_z = []

    def _rem_key(self, d, key):
        r = dict(d)
        del r[key]
        return r

    def _sweep(self, v, f, reference=""):

        keys = list(v)

        if len(keys) > 1:
            vals = v[keys[0]]
            others = v
            others = self._rem_key(others, keys[0])
            for val in vals:
                all_params = dict(f)
                all_params[keys[0]] = val

                self._sweep(
                    others,
                    all_params,
                    reference="%s_%s%s"
                    % (reference, keys[0], (str(val)).replace("-", "min")),
                )

        else:
            vals = v[keys[0]]
            for val in vals:
                all_params = dict(f)
                all_params[keys[0]] = val
                r = "%s_%s%s" % (reference, keys[0], (str(val)).replace("-", "min"))
                ref_here = "RUN%s_%s_%s" % (self.index, self.sim.id, r)
                self.index += 1
                all_params["reference"] = ref_here

                self.report["Simulations"][ref_here] = OrderedDict()
                report_here = self.report["Simulations"][ref_here]

                report_here["parameters"] = all_params

    def _get_sim_duration_ms(self, parameters):
        if "duration" in parameters:
            return parameters["duration"]
        else:
            return self.sim.duration

    def _run_all(self):

        import pp

        ppservers = ()
        job_server = pp.Server(self.num_parallel_runs, ppservers=ppservers)
        print_v(
            "\n  == Running %i jobs across %i local processes\n "
            % (self.total_todo, job_server.get_ncpus())
        )
        jobs = []
        job_refs = {}

        submitted = 0
        for ref in self.report["Simulations"]:

            report_here = self.report["Simulations"][ref]

            params = report_here["parameters"]
            print_v("---- Submitting %s: %s" % (ref, params))

            job_dir = os.path.join(self.result_dir, ref)
            os.mkdir(job_dir)

            vars = (self.runner, submitted + 1, self.total_todo, job_dir, params)

            job = job_server.submit(
                run_instance,
                args=vars,
                modules=(
                    "pyneuroml.pynml",
                    "shutil",
                    "neuroml",
                    "neuromllite",
                    "neuromllite.sweep.ParameterSweep",
                    "neuromllite.utils",
                ),
            )
            jobs.append(job)
            job_refs[len(jobs) - 1] = ref

            submitted += 1

        print_v("Submitted all jobs: %s" % job_refs)

        for job_i in range(len(jobs)):
            job = jobs[job_i]
            ref = job_refs[job_i]
            report_here = self.report["Simulations"][ref]
            report_here["analysis"] = OrderedDict()
            params = report_here["parameters"]

            print_v("Checking parallel job %i/%i (%s)" % (job_i, len(jobs), ref))
            traces, events = job()

            self.last_network_ran = None

            if len(traces) > 0:
                times = [t * 1000.0 for t in traces["t"]]
                volts = OrderedDict()
                for tr in traces:
                    if tr.endswith("/v"):
                        volts[tr] = [v * 1000.0 for v in traces[tr]]
                    if tr.endswith("/r"):
                        volts[tr] = [r for r in traces[tr]]
                    if tr.endswith("/V"):
                        volts[tr] = [V for V in traces[tr]]

                print_v("Analysing %s..." % traces.keys())

                analysis_data = analysis.NetworkAnalysis(
                    volts,
                    times,
                    self.analysis_var,
                    start_analysis=0,
                    end_analysis=times[-1],
                    smooth_data=False,
                    show_smoothed_data=False,
                    verbose=self.verbose,
                )

                analysed = analysis_data.analyse()

                for a in sorted(analysed.keys()):
                    ref0, var = a.split(":")
                    if not ref0 in report_here["analysis"]:
                        report_here["analysis"][ref0] = OrderedDict()
                    report_here["analysis"][ref0][var] = analysed[a]

            for e in sorted(events.keys()):
                x = events[e]
                print_v(
                    "Examining event %s: %s -> %s (len: %i)"
                    % (
                        e,
                        x[0] if len(x) > 0 else "-",
                        x[-1] if len(x) > 0 else "-",
                        len(x),
                    )
                )
                ref0 = "%s/spike" % e
                analysed = OrderedDict()
                l = len(x)
                tmax_si = self._get_sim_duration_ms(params) / 1000.0
                f_hz = l / tmax_si
                # print_v('This has %s points in %s sec, so %s Hz'%(l,tmax_si, f_hz))
                analysed["mean_spike_frequency"] = f_hz

                if not ref0 in report_here["analysis"]:
                    report_here["analysis"][ref0] = OrderedDict()

                report_here["analysis"][ref0] = analysed

            if self.plot_all or self.heatmap_all:
                for y in traces.keys():
                    if y != "t":
                        pop_id = y.split("[")[0] if "[" in y else y.split("/")[0]
                        pop = None  # self.last_network_ran.get_child(pop_id, 'populations')
                        if pop:
                            color = [float(c) for c in pop.properties["color"].split()]
                            # print_v("This trace %s has population %s: %s, so color: %s"%(y,pop_id,pop,color))
                        else:
                            # print_v("This trace %s has population %s: %s which has no color..."%(y,pop_id,pop))
                            color = [1, 0, 0]

                        if self.plot_all:
                            label = "%s (%s)" % (y, params)
                            self.ax.plot(
                                [t * 1000 for t in traces["t"]],
                                [v * 1000 for v in traces[y]],
                                label=label,
                            )

                        if self.heatmap_all:
                            dt = self.sim.dt if not "dt" in params else params["dt"]
                            downscale = int(0.1 / dt)
                            d = [
                                traces[y][i] * 1000
                                for i in range(len(traces[y]))
                                if i % downscale == 0
                            ]
                            tt = [
                                traces["t"][i] * 1000
                                for i in range(len(traces["t"]))
                                if i % downscale == 0
                            ]

                            param_name = self.vary.keys()[0]
                            pval = get_value_in_si(params[param_name])
                            if self.hm_x == None:
                                self.hm_x = tt
                            print_v(
                                "  ==  Trace %s (%s) downscaled by factor %i from %i to %i points for heatmap; y value: %s=%s"
                                % (
                                    y,
                                    ref,
                                    downscale,
                                    len(traces[y]),
                                    len(d),
                                    param_name,
                                    pval,
                                )
                            )
                            self.hm_y.append(pval)
                            self.hm_z.append(d)

            print_v(
                "Finished checking parallel job %i/%i (%s)" % (job_i, len(jobs), ref)
            )

        job_server.print_stats()
        job_server.destroy()
        print_v("-------------------------------------------")

    def run(self):

        print_v("Running...")
        self._sweep(self.vary, self.fixed)
        self._run_all()

        if self.plot_all and self.save_plot_all_to:
            print_v(
                "Saving image to %s of plot" % (os.path.abspath(self.save_plot_all_to))
            )
            plt.savefig(self.save_plot_all_to, bbox_inches="tight")

        if self.heatmap_all:

            self.hm_fig, self.hm_ax = plt.subplots()
            z = np.array(self.hm_z)

            print_v(
                "Plotting x: %s->%s (%i), y: %s->%s (%i), z: %s->%s (%i)"
                % (
                    self.hm_x[0],
                    self.hm_x[-1],
                    len(self.hm_x),
                    self.hm_y[0],
                    self.hm_y[-1],
                    len(self.hm_y),
                    z.min(),
                    z.max(),
                    z.size,
                )
            )

            # yvals = [i for i in range(len(self.hm_y))]
            # yvals = np.array(self.hm_y)

            if not self.heatmap_lims:
                plot0 = self.hm_ax.pcolormesh(
                    np.array(self.hm_x), np.array(self.hm_y), z, cmap=self.colormap
                )
            else:
                plot0 = self.hm_ax.pcolormesh(
                    np.array(self.hm_x),
                    np.array(self.hm_y),
                    z,
                    vmin=self.heatmap_lims[0],
                    vmax=self.heatmap_lims[1],
                    cmap=self.colormap,
                )

            # plt.set_yticks(np.arange(a_n_.shape[0]) + 0.5, minor=False)
            # self.hm_ax.set_yticklabels(['ff%s'%i for i in range(len(self.hm_y))])

            plt.xlabel("Time (ms)")
            plt.ylabel("%s " % self.vary.keys()[0])
            plt.xlim([self.hm_x[0], self.hm_x[-1]])
            plt.ylim([self.hm_y[0], self.hm_y[-1]])
            title = "Values of %s" % self.vary.keys()
            self.hm_fig.canvas.set_window_title(title)
            plt.title(title)
            # plt.axes().set_aspect('equal')

            cb1 = self.hm_fig.colorbar(plot0)
            cb1.set_label("Membrane potential (mV)")

            if self.save_heatmap_to:
                print_v(
                    "Saving image to %s of plot"
                    % (os.path.abspath(self.save_heatmap_to))
                )
                plt.savefig(self.save_heatmap_to, bbox_inches="tight")

        if self.show_plot_already:
            plt.show()

        return self.report

    def print_report(self):

        print_v("--- REPORT:")
        import json

        print_v(json.dumps(self.report, indent=4))

    def plotLines(
        self,
        first_param,
        value,
        second_param=None,
        save_figure_to=None,
        logx=False,
        logy=False,
    ):

        all_pvals = OrderedDict()
        all_lines = OrderedDict()

        all_traces = []

        DEFAULT_TRACE = self.sim.id
        if not second_param:
            all_traces = [DEFAULT_TRACE]
        else:
            for ref, info in self.report["Simulations"].items():
                val2 = get_value_in_si(info["parameters"][second_param])
                trace_id = "%s__%s" % (second_param, val2)
                trace_id = val2
                if not trace_id in all_traces:
                    all_traces.append(trace_id)

        for t in all_traces:
            all_lines[t] = {}
            all_pvals[t] = {}

        for ref, info in self.report["Simulations"].items():
            print_v("Checking %s: %s" % (ref, info["parameters"]))

            pval = get_value_in_si(info["parameters"][first_param])

            if not second_param:
                trace_id = DEFAULT_TRACE
            else:
                val2 = get_value_in_si(info["parameters"][second_param])
                trace_id = val2

            if ":" in value:
                matching_ref = value.split(":")[0]
                feature = value.split(":")[1]
            else:
                matching_ref = "*"
                feature = value

            for cell_ref in info["analysis"]:
                print_v(
                    "Checking if %s matches %s, feature: %s (from %s)"
                    % (cell_ref, matching_ref, feature, value)
                )
                if matching_ref == cell_ref or matching_ref == "*":
                    # print('y')
                    if not cell_ref in all_lines[trace_id]:
                        all_lines[trace_id][cell_ref] = []
                        all_pvals[trace_id][cell_ref] = []

                    vval = info["analysis"][cell_ref][feature]

                    all_lines[trace_id][cell_ref].append(vval)
                    all_pvals[trace_id][cell_ref].append(pval)

        print_v("Plot x: %s" % all_pvals)
        print_v("Plot y: %s" % all_lines)

        xs = []
        ys = []
        labels = []
        markers = []
        colors = []
        maxy = -1 * sys.float_info.max

        for t in all_traces:

            for ref in all_lines[t]:
                print_v("Add data %s, %s" % (t, ref))

                xs.append(all_pvals[t][ref])
                ys.append(all_lines[t][ref])

                maxy = max(maxy, max(all_lines[t][ref]))

                labels.append("%s - %s" % (t, ref))
                markers.append("o")

                pop_id = ref.split("[")[0] if "[" in ref else ref.split("/")[0]
                if self.last_network_ran:
                    pop = self.last_network_ran.get_child(pop_id, "populations")
                    color = [float(c) for c in pop.properties["color"].split()]
                else:
                    color = [random.random(), random.random(), random.random()]
                    pop = None
                print_v(
                    "This trace %s has population %s: %s, so color: %s"
                    % (ref, pop_id, pop, color)
                )
                colors.append(color)

        xlim = None
        ylim = None

        yaxis = value.replace("_", " ")
        yaxis = yaxis[0].upper() + yaxis[1:]

        if value == "mean_spike_frequency":
            yaxis += " (Hz)"
            ylim = [maxy * -0.1, maxy * 1.1]
            print_v("Setting y axes on freq plot to: %s" % ylim)

        ax = pynml.generate_plot(
            xs,
            ys,
            "Plot of %s vs %s in %s" % (value, first_param, self.sim),
            xaxis=first_param,
            yaxis=yaxis,
            labels=labels if len(labels) > 1 else None,
            markers=markers,
            colors=colors,
            xlim=xlim,
            ylim=ylim,
            logx=logx,
            logy=logy,
            show_plot_already=False,
            legend_position="right",
            save_figure_to=save_figure_to,
        )  # Save figure


def run_instance(runner, i, total, job_dir, params):

    print(
        "============================================================= \n\n"
        + "     Instance (%s of %s): %s\n" % (i, total, params)
    )

    return runner.run_once(job_dir, **params)


"""
    Loads a single NeuroMLlite simulation/network, allows instances of this to be generated in
    multiple supported simulators, potentially with some parameters changed, and can plot
    the activity of the cells in the simulations
"""


class NeuroMLliteRunner:
    def __init__(self, nmllite_sim, simulator="jNeuroML"):

        real_sim = os.path.realpath(nmllite_sim)
        print_v("Created NeuroMLliteRunner to run %s in %s" % (real_sim, simulator))

        self.base_dir = os.path.dirname(real_sim)
        self.nmllite_sim = nmllite_sim
        self.simulator = simulator

    """
        Run a single instance of the simulation, changing the parameters specified
    """

    def run_once(self, job_dir, **kwargs):

        from neuromllite.utils import print_v
        from neuromllite.utils import load_simulation_json, load_network_json
        from neuromllite.NetworkGenerator import generate_and_run
        from pyneuroml.pynml import get_value_in_si

        print_v("Running NeuroMLlite simulation in dir: %s..." % job_dir)
        sim = load_simulation_json(self.nmllite_sim)
        import random

        sim.id = "%s%s" % (
            sim.id,
            "_%s" % kwargs["reference"] if "reference" in kwargs else "",
        )
        network = load_network_json(self.base_dir + "/" + sim.network)

        for a in kwargs:
            if a in network.parameters:
                print_v("  Setting %s to %s in network..." % (a, kwargs[a]))
                network.parameters[a] = kwargs[a]
            elif a in sim.fields:
                print_v("  Setting %s to %s in simulator..." % (a, kwargs[a]))
                setattr(sim, a, kwargs[a])
            else:
                print_v(
                    "  Cannot set parameter %s to %s in network or simulator..."
                    % (a, kwargs[a])
                )

        traces, events = generate_and_run(
            sim,
            simulator=self.simulator,
            network=network,
            return_results=True,
            base_dir=self.base_dir,
            target_dir=job_dir,
        )

        print_v("Returned traces: %s, events: %s" % (traces.keys(), events.keys()))

        return traces, events


if __name__ == "__main__":

    if "-2d" in sys.argv:
        fixed = {"dt": 0.025}
        vary = {
            "stim_amp": ["%spA" % (i) for i in xrange(-40, 220, 40)],
            "stim_del": ["%sms" % (i) for i in xrange(10, 40, 10)],
        }

        vary = {
            "stim_amp": ["%spA" % (i) for i in [-50, 100]],
            "stim_del": ["%sms" % (i) for i in [10, 20]],
        }

        vary = {
            "stim_amp": ["%spA" % (i) for i in xrange(-40, 220, 20)],
            "stim_del": ["%sms" % (i) for i in [10, 20]],
        }

        simulator = "jNeuroML_NEURON"
        simulator = "jNeuroML"

        nmllr = NeuroMLliteRunner("Sim_HHTest.json", simulator=simulator)

        ps = ParameterSweep(
            nmllr,
            vary,
            fixed,
            num_parallel_runs=6,
            plot_all=True,
            heatmap_all=False,
            show_plot_already=False,
        )

        report = ps.run()
        ps.print_report()

        #  ps.plotLines('weightInput','average_last_1percent',save_figure_to='average_last_1percent.png')
        # ps.plotLines('weightInput','mean_spike_frequency',save_figure_to='mean_spike_frequency.png')
        ps.plotLines(
            "stim_amp",
            value="pop_hhcell[0]/v:mean_spike_frequency",
            second_param="stim_del",
            save_figure_to="mean_spike_frequency_hh.png",
        )
        """
        ps.plotLines('stim_amp',
                     value='pop_hhcell[0]/v:first_spike_time',
                     second_param='stim_del',
                     save_figure_to='first_spike_time_hh.png')"""
        ps.plotLines(
            "stim_amp",
            value="pop_hhcell[0]/v:maximum",
            second_param="stim_del",
            save_figure_to="maximum_hh.png",
        )

        import matplotlib.pyplot as plt

        plt.show()

    elif "-hh" in sys.argv:
        fixed = {"dt": 0.025}

        vary = {"stim_amp": ["%spA" % (i) for i in xrange(-200, 1500, 10)]}
        vary = {"stim_amp": ["%spA" % (i) for i in xrange(-100, 1800, 20)]}
        # vary = {'stim_amp':['%spA'%(i) for i in xrange(-100,500,5)]}

        simulator = "jNeuroML_NetPyNE"
        simulator = "NetPyNE"
        simulator = "jNeuroML_NEURON"
        simulator = "jNeuroML"

        nmllr = NeuroMLliteRunner("Sim_HHTest.json", simulator=simulator)

        ps = ParameterSweep(
            nmllr,
            vary,
            fixed,
            num_parallel_runs=6,
            plot_all=True,
            heatmap_all=True,
            heatmap_lims=[-100, 20],
            show_plot_already=False,
        )

        report = ps.run()
        ps.print_report()

        ps.plotLines(
            "stim_amp",
            "mean_spike_frequency",
            save_figure_to="mean_spike_frequency_hh.png",
        )

        import matplotlib.pyplot as plt

        plt.show()

    elif "-dt" in sys.argv:
        fixed = {"stim_amp": "500pA"}

        vary = {"dt": [0.025, 0.02, 0.015, 0.01, 0.005, 0.0025]}
        vary = {"dt": [0.1, 0.05, 0.025, 0.01, 0.005, 0.0025, 0.001]}

        simulator = "jNeuroML_NetPyNE"
        simulator = "NetPyNE"
        simulator = "jNeuroML"
        simulator = "jNeuroML_NEURON"

        nmllr = NeuroMLliteRunner("Sim_HHTest.json", simulator=simulator)

        ps = ParameterSweep(
            nmllr,
            vary,
            fixed,
            num_parallel_runs=6,
            save_plot_all_to="dt_traces_hh.png",
            heatmap_all=True,
            save_heatmap_to="heatmap_dt_hh.png",
            plot_all=True,
            show_plot_already=False,
        )

        report = ps.run()
        ps.print_report()

        ps.plotLines(
            "dt",
            "mean_spike_frequency",
            save_figure_to="mean_spike_frequency_dt_hh.png",
            logx=True,
        )

        import matplotlib.pyplot as plt

        plt.show()

    elif "-run" in sys.argv:

        simulator = "jNeuroML_NetPyNE"
        simulator = "jNeuroML"
        # simulator = 'PyNN_NEST'
        nmllr = NeuroMLliteRunner(
            "../../examples/SimExample7.json", simulator=simulator
        )

        traces, events = nmllr.run_once(".", reference="ref0")

    elif "-hr" in sys.argv:
        fixed = {"dt": 25}

        quick = False
        # quick=True

        vary = {"c": [-3, -1, 1, 3]}
        # vary = {'a':[.8,1,1.2]}
        # vary = {'eta':['100Hz']}
        # vary = {'stim_amp':['1.5pA']}

        simulator = "jNeuroML"
        simulator = "jNeuroML_NEURON"
        nmllr = NeuroMLliteRunner(
            "../../examples/SimExample9.json", simulator=simulator
        )

        if quick:
            pass

        ps = ParameterSweep(
            nmllr,
            vary,
            fixed,
            num_parallel_runs=6,
            plot_all=True,
            heatmap_all=False,
            show_plot_already=False,
        )

        report = ps.run()
        ps.print_report()

        # for k in vary:
        #    ps.plotLines(k,'mean_spike_frequency',save_figure_to='mean_spike_frequency_dt_hh.png', logx=True)

        #  ps.plotLines('weightInput','average_last_1percent',save_figure_to='average_last_1percent.png')
        # ps.plotLines('weightInput','mean_spike_frequency',save_figure_to='mean_spike_frequency.png')
        # ps.plotLines('eta','mean_spike_frequency',save_figure_to='mean_spike_frequency.png')

        import matplotlib.pyplot as plt

        plt.show()

    else:
        fixed = {"dt": 0.025, "N": 10}

        quick = False
        # quick=True

        vary = {"stim_amp": ["%spA" % (i / 10.0) for i in xrange(-10, 20, 2)]}
        vary = {"stim_amp": ["%spA" % (i / 10.0) for i in xrange(-10, 20, 5)]}
        vary = {"weightInput": [1, 2, 5, 10]}
        vary = {"weightInput": [1, 2, 3, 20]}
        vary = {"eta": [i / 100.0 for i in xrange(0, 200, 20)]}
        # vary = {'eta':['100Hz']}
        # vary = {'stim_amp':['1.5pA']}

        nmllr = NeuroMLliteRunner("../../examples/SimExample7.json")

        if quick:
            pass

        ps = ParameterSweep(
            nmllr,
            vary,
            fixed,
            num_parallel_runs=6,
            plot_all=False,
            heatmap_all=False,
            show_plot_already=False,
        )

        report = ps.run()
        ps.print_report()

        #  ps.plotLines('weightInput','average_last_1percent',save_figure_to='average_last_1percent.png')
        # ps.plotLines('weightInput','mean_spike_frequency',save_figure_to='mean_spike_frequency.png')
        ps.plotLines(
            "eta", "mean_spike_frequency", save_figure_to="mean_spike_frequency.png"
        )

        import matplotlib.pyplot as plt

        plt.show()



#
#
#   Helper class for parsing connectivity data for structuring data on weights etc.
#
#

from neuromllite.utils import print_v
from neuromllite.DefaultNetworkHandler import DefaultNetworkHandler
from pyneuroml.pynml import convert_to_units


class ConnectivityHandler(DefaultNetworkHandler):

    CUTOFF_INH_SYN_MV = -50  # erev below -50mV => inhibitory, above => excitatory

    include_ext_inputs = False

    positions = {}

    pop_sizes = {}
    pop_colors = {}
    pop_types = {}
    pop_nml_component_objs = {}

    proj_weights = {}
    proj_shapes = {}
    proj_lines = {}
    proj_pre_pops = {}
    proj_post_pops = {}
    proj_types = {}
    proj_conns = {}
    proj_tot_weight = {}
    proj_syn_objs = {}
    proj_individual_weights = {}
    proj_individual_scaled_weights = {}
    proj_individual_conn_numbers = {}
    proj_delays = {}

    syn_conds_used = {}

    sizes_ils = {}
    pops_ils = {}
    weights_ils = {}
    input_comp_obj_ils = {}

    def handle_document_start(self, id, notes):

        print_v("Document: %s" % id)

    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        self.network_id = network_id

    # Nice clean number for use in graphs, labels etc.
    def format_float(self, f, d=3, approx=False):
        if int(f) == f:
            return int(f)

        template = "%%.%if" % d  # e.g. '%.2f'

        ff = float(template % f)
        if f == ff:
            return "%s" % ff
        return "%s%s" % ("~" if approx else "", ff)

    # Is the graph to be generated at the level of populations or individual cells?
    def is_cell_level(self):
        return self.level <= 0

    def get_cell_identifier(self, population, index):
        size = self.pop_sizes[population]
        i = "%s%i" % ("0" * (len(str(size - 1)) - len(str(index))), index)
        return "%s_%s" % (population, i)

    def _get_proj_class(self, proj_type):
        if proj_type == "excitatory" or proj_type == "inhibitory":
            return "chemical"
        elif proj_type == "excitatorycontinuous" or proj_type == "inhibitorycontinuous":
            return "continuous"
        else:
            return proj_type

    def get_size_pre_pop(self, projName):
        return self.pop_sizes[self.proj_pre_pops[projName]]

    def get_size_post_pop(self, projName):
        return self.pop_sizes[self.proj_post_pops[projName]]

    def get_reversal_potential_mV(self, synapse_obj):

        if hasattr(synapse_obj, "erev"):
            return convert_to_units(synapse_obj.erev, "mV")

        elif hasattr(synapse_obj, "e_rev"):  # PyNN, no units, in mV
            return float(synapse_obj.e_rev)

        else:
            return None

    def _get_gbase_nS(self, projName, return_orig_string_also=False):

        gbase_nS = None
        gbase = "???"
        # print('Getting gbase for %s'%projName)
        if projName in self.proj_syn_objs:
            syn = self.proj_syn_objs[projName]
            if hasattr(syn, "gbase"):
                gbase = syn.gbase
                gbase_nS = convert_to_units(gbase, "nS")
            elif hasattr(syn, "conductance"):
                gbase = syn.conductance
                gbase_nS = convert_to_units(gbase, "nS")

            self.syn_conds_used[syn.id] = "%s nS (%s)" % (gbase_nS, gbase)

        else:
            self.syn_conds_used[projName] = "Syn for %s not found..." % projName

        if return_orig_string_also:
            return gbase_nS, gbase

        return gbase_nS

    def _scale_individual_weight(self, weight, projName):

        orig_weight = weight

        if self.scale_by_post_pop_cond:
            gbase_nS = self._get_gbase_nS(projName)
            if gbase_nS != None:
                weight *= gbase_nS

        if not orig_weight == weight:
            # print(' - Weight for %s modified %s->%s'%(projName, orig_weight, weight))
            pass

        return weight

    def _scale_population_weight(self, weight, projName):

        orig_weight = weight
        if self.scale_by_post_pop_size:
            weight /= self.get_size_post_pop(projName)

        if self.scale_by_post_pop_cond:
            gbase_nS = self._get_gbase_nS(projName)
            if gbase_nS != None:
                weight *= gbase_nS

        if not orig_weight == weight:
            # print(' - Weight for %s modified %s->%s'%(projName, orig_weight, weight))
            pass

        return weight

    def handle_location(self, id, population_id, component, x, y, z):

        pass

    def handle_connection(
        self,
        projName,
        id,
        prePop,
        postPop,
        synapseType,
        preCellId,
        postCellId,
        preSegId=0,
        preFract=0.5,
        postSegId=0,
        postFract=0.5,
        delay=0,
        weight=1.0,
    ):

        # print_v(" - Connection for %s, cell %i -> %i, w: %s"%(projName, preCellId, postCellId, weight))
        self.proj_conns[projName] += 1
        self.proj_tot_weight[projName] += weight
        if self.is_cell_level():
            self.proj_individual_weights[projName][preCellId][postCellId] += weight
            self.proj_individual_scaled_weights[projName][preCellId][
                postCellId
            ] += self._scale_individual_weight(weight, projName)
            self.proj_individual_conn_numbers[projName][preCellId][postCellId] += 1
            """
            print_v("   - Now total weight between these cells is %s (scaled: %s) from %i individual conns" % \
                       (self.proj_individual_weights[projName][preCellId][postCellId],
                       self.proj_individual_scaled_weights[projName][preCellId][postCellId],
                       self.proj_individual_conn_numbers[projName][preCellId][postCellId]))"""
            self.proj_delays[projName][preCellId][postCellId] = delay

    def finalise_projection(
        self, projName, prePop, postPop, synapse=None, type="projection"
    ):

        # weight = self.proj_tot_weight[projName]
        # self.max_weight = max(self.max_weight, weight)
        # self.min_weight = min(self.min_weight, weight)
        # print_v("Now weights range %s->%s"%(self.min_weight, self.max_weight))
        pass
        # print_v("Projection finalising: "+projName+" from "+prePop+" to "+postPop+" completed")

    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):
        if self.include_ext_inputs:
            # self.print_input_information('INIT:  '+inputListId, population_id, component, size)
            self.sizes_ils[inputListId] = 0
            self.pops_ils[inputListId] = population_id
            self.input_comp_obj_ils[inputListId] = input_comp_obj
            self.weights_ils[inputListId] = 0

    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):
        if self.include_ext_inputs:
            self.sizes_ils[inputListId] += 1
            self.weights_ils[inputListId] += weight



from neuromllite import Network, Cell, Synapse, NetworkReader, InputSource, Input
from neuromllite.NetworkGenerator import generate_neuroml2_from_network
import sys


################################################################################
###   Build new network

percent = 5
net = Network(
    id="BBP_%spercent" % percent,
    notes="A network with the Blue Brain Project connectivity data (%s% of total cells)",
)

default_cell = Cell(id="hhcell", neuroml2_source_file="test_files/hhcell.cell.nml")

net.network_reader = NetworkReader(
    type="BBPConnectomeReader",
    parameters={
        "id": net.id,
        "filename": "test_files/cons_locs_pathways_mc0_Column.h5",
        "percentage_cells_per_pop": percent,
        "DEFAULT_CELL_ID": default_cell.id,
        "cell_info": {default_cell.id: default_cell},
    },
)

net.cells.append(default_cell)
net.synapses.append(
    Synapse(id="ampa", neuroml2_source_file="test_files/ampa.synapse.nml")
)
net.synapses.append(
    Synapse(id="gaba", neuroml2_source_file="test_files/gaba.synapse.nml")
)


input_source = InputSource(
    id="poissonFiringSyn", neuroml2_source_file="test_files/inputs.nml"
)
net.input_sources.append(input_source)

for pop in ["L4_PC"]:
    net.inputs.append(
        Input(
            id="stim_%s" % pop,
            input_source=input_source.id,
            population=pop,
            percentage=80,
        )
    )

new_file = net.to_json_file("%s.json" % net.id)


################################################################################
###   Builds a NeuroML 2 representation, save as XML

format_ = "xml"
nml_file_name, nml_doc = generate_neuroml2_from_network(
    net,
    nml_file_name="%s.net.nml%s" % (net.id, ".h5" if format_ == "hdf5" else ""),
    format=format_,
)


from neuromllite import Simulation

record_traces = {"all": "*"}
record_spikes = {"all": "*"}


sim = Simulation(
    id="SimExample5",
    network=new_file,
    duration=500,
    dt=0.025,
    record_traces=record_traces,
    record_spikes=record_spikes,
)

sim.to_json_file()


################################################################################
###   Run in some simulators

from neuromllite.NetworkGenerator import check_to_generate_or_run
import sys

check_to_generate_or_run(sys.argv, sim)



import os
import sys
from neuromllite.NetworkGenerator import check_to_generate_or_run

os.chdir("..")
sys.path.append(".")

print("Running PyNN Ex7 test script with %s" % sys.argv)

if "nest" in sys.argv:
    sys.argv.append("-pynnnest")
if "neuron" in sys.argv:
    sys.argv.append("-pynnnrn")
if "brian" in sys.argv:
    sys.argv.append("-pynnbrian")

from Example7 import generate

sim, net = generate()

check_to_generate_or_run(sys.argv, sim)



#
#
#   A class to write to the Sonata format...
#
#

from neuromllite.utils import print_v
from neuromllite.DefaultNetworkHandler import DefaultNetworkHandler

import h5py
import numpy as np
from neuromllite.utils import save_to_json_file
import os


class SonataHandler(DefaultNetworkHandler):

    positions = {}
    pop_indices = {}

    DEFAULT_NODE_GROUP_ID = 0
    pop_type_ids = {}
    node_type_csv_info = {}

    input_info = {}

    def __init__(self):
        print_v("Initiating Sonata handler")

    def handle_document_start(self, id, notes):

        print_v("Parsing for Sonata export: %s" % id)

        self.config_file_info = {}
        self.config_file_info["network"] = "./circuit_config.json"
        self.config_file_info["simulation"] = "./simulation_config.json"

        self.circuit_file_info = {}
        self.circuit_file_info["manifest"] = {}
        self.circuit_file_info["manifest"]["$NETWORK_DIR"] = "./network"
        if not os.path.exists(self.circuit_file_info["manifest"]["$NETWORK_DIR"]):
            os.mkdir(self.circuit_file_info["manifest"]["$NETWORK_DIR"])

        self.circuit_file_info["manifest"]["$COMPONENT_DIR"] = "./components"
        if not os.path.exists(self.circuit_file_info["manifest"]["$COMPONENT_DIR"]):
            os.mkdir(self.circuit_file_info["manifest"]["$COMPONENT_DIR"])

        self.circuit_file_info["components"] = {}
        self.circuit_file_info["components"][
            "synaptic_models_dir"
        ] = "$COMPONENT_DIR/synaptic_models"

        if not os.path.exists("./components/synaptic_models"):
            os.mkdir("./components/synaptic_models")

        self.circuit_file_info["components"][
            "point_neuron_models_dir"
        ] = "$COMPONENT_DIR/point_neuron_models_dir"
        if not os.path.exists("./components/point_neuron_models_dir"):
            os.mkdir("./components/point_neuron_models_dir")

        self.circuit_file_info["networks"] = {}
        self.circuit_file_info["networks"]["nodes"] = []
        self.circuit_file_info["networks"]["nodes"].append({})

    def finalise_document(self):

        print_v("Writing file...: %s" % id)
        self.sonata_nodes.close()

        node_type_filename = "%s_node_types.csv" % self.network_id

        self.circuit_file_info["networks"]["nodes"][0]["node_types_file"] = (
            "$NETWORK_DIR/%s" % node_type_filename
        )

        node_type_file = open(
            os.path.join(
                self.circuit_file_info["manifest"]["$NETWORK_DIR"], node_type_filename
            ),
            "w",
        )
        header = ""
        for var in self.node_type_csv_info[self.node_type_csv_info.keys()[0]]:
            header += "%s " % var
        node_type_file.write(header + "\n")

        for pop_id in self.node_type_csv_info:
            line = ""
            for var in self.node_type_csv_info[pop_id]:
                line += "%s " % self.node_type_csv_info[pop_id][var]
            node_type_file.write(line + "\n")

        node_type_file.close()

        save_to_json_file(self.config_file_info, "config.json", indent=2)
        save_to_json_file(self.circuit_file_info, "circuit_config.json", indent=2)

    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        self.network_id = network_id

        if temperature:
            print_v("  Temperature: " + temperature)
        if notes:
            print_v("  Notes: " + notes)

        nodes_filename = "%s_nodes.sonata.h5" % network_id
        self.circuit_file_info["networks"]["nodes"][0]["nodes_file"] = (
            "$NETWORK_DIR/%s" % nodes_filename
        )

        self.sonata_nodes = h5py.File(
            os.path.join(
                self.circuit_file_info["manifest"]["$NETWORK_DIR"], nodes_filename
            ),
            "w",
        )
        self.sonata_nodes.attrs.create("version", [0, 1], dtype=np.uint32)
        self.sonata_nodes.attrs.create("magic", np.uint32(0x0A7A))

    def handle_population(
        self, population_id, component, size=-1, component_obj=None, properties={}
    ):

        sizeInfo = " as yet unspecified size"
        if size >= 0:
            sizeInfo = ", size: " + str(size) + " cells"
        if component_obj:
            compInfo = " (%s)" % component_obj.__class__.__name__
        else:
            compInfo = ""

        print_v(
            "Population: "
            + population_id
            + ", component: "
            + component
            + compInfo
            + sizeInfo
        )

        self.sonata_nodes.create_group("nodes/%s" % population_id)
        self.sonata_nodes.create_group("nodes/%s/0" % population_id)

        node_type_id = 100 + len(self.pop_type_ids)
        self.pop_type_ids[population_id] = node_type_id

        self.node_type_csv_info[population_id] = {}
        self.node_type_csv_info[population_id]["node_type_id"] = node_type_id
        self.node_type_csv_info[population_id]["pop_name"] = population_id
        self.node_type_csv_info[population_id]["model_name"] = component
        ##self.node_type_csv_info[population_id]['location'] = '???'
        self.node_type_csv_info[population_id]["model_template"] = "nest:iaf_psc_alpha"
        self.node_type_csv_info[population_id]["model_type"] = "point_process"
        self.node_type_csv_info[population_id]["dynamics_params"] = (
            "%s.json" % component
        )

    def handle_location(self, id, population_id, component, x, y, z):

        if not population_id in self.positions:
            self.positions[population_id] = np.array([[x, y, z]])
            self.pop_indices[population_id] = np.array([id])
        else:
            self.positions[population_id] = np.concatenate(
                (self.positions[population_id], [[x, y, z]])
            )
            self.pop_indices[population_id] = np.concatenate(
                (self.pop_indices[population_id], [id])
            )

    def finalise_population(self, population_id):

        self.sonata_nodes.create_dataset(
            "nodes/%s/0/positions" % population_id, data=self.positions[population_id]
        )
        self.sonata_nodes.create_dataset(
            "nodes/%s/node_group_index" % population_id,
            data=self.pop_indices[population_id],
        )
        self.sonata_nodes.create_dataset(
            "nodes/%s/node_group_id" % population_id,
            data=[self.DEFAULT_NODE_GROUP_ID for i in self.pop_indices[population_id]],
        )
        self.sonata_nodes.create_dataset(
            "nodes/%s/node_id" % population_id, data=self.pop_indices[population_id]
        )

        self.sonata_nodes.create_dataset(
            "nodes/%s/node_type_id" % population_id,
            data=[
                self.pop_type_ids[population_id]
                for i in self.pop_indices[population_id]
            ],
        )

    #
    #  Should be overridden to create input source array
    #
    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):

        self.print_input_information(inputListId, population_id, component, size)

        if size < 0:
            self.log.error(
                "Error! Need a size attribute in sites element to create spike source!"
            )
            return

        self.input_info[inputListId] = (population_id, component, [])

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):

        print_v(
            "Input: %s[%s], cellId: %i, seg: %i, fract: %f, weight: %f"
            % (inputListId, id, cellId, segId, fract, weight)
        )

        self.input_info[inputListId][2].append(cellId)

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def finalise_input_source(self, inputName):
        print_v("Input : %s completed" % inputName)



#
#
#   A class to handle events by PyNN...
#
#

from neuromllite.utils import print_v
from neuromllite.utils import evaluate
from neuromllite.DefaultNetworkHandler import DefaultNetworkHandler

from pyneuroml.pynml import convert_to_units

from importlib import import_module


class PyNNHandler(DefaultNetworkHandler):

    populations = {}
    projections = {}
    input_sources = {}
    input_info = {}

    def __init__(self, simulator, dt, reference):
        print_v("Initiating PyNN with simulator %s" % simulator)
        if simulator == "nest":
            import nest  # temp
        self.sim = import_module("pyNN.%s" % simulator)
        self.dt = dt
        self.sim.setup(
            timestep=self.dt, debug=True, reference=reference, save_format="xml"
        )

    def set_cells(self, cells):
        self.cells = cells

    def set_receptor_types(self, receptor_types):
        self.receptor_types = receptor_types

    def add_input_source(self, input_source, network):
        input_params = input_source.parameters if input_source.parameters else {}

        for ip in input_params:
            input_params[ip] = evaluate(input_params[ip], network.parameters)

        """ This is a quick hack to support noisyCurrentSource before that type is integrated into the 
            core of NeuroML...
            TODO: remove when integrated!
        """
        if input_source.lems_source_file and "noisyCurrentSource" in input_source.id:
            pynn_input_params = {}
            for p in input_params:
                if p == "delay":
                    pynn_input_params["start"] = convert_to_units(input_params[p], "ms")
                elif p == "duration":
                    pynn_input_params["stop"] = convert_to_units(
                        input_params[p], "ms"
                    ) + convert_to_units(input_params["delay"], "ms")
                elif p == "mean":
                    pynn_input_params["mean"] = convert_to_units(input_params[p], "nA")
                elif p == "stdev":
                    pynn_input_params["stdev"] = convert_to_units(input_params[p], "nA")
                elif p == "noiseDt":
                    pynn_input_params["dt"] = convert_to_units(input_params[p], "ms")
                else:
                    raise Exception(
                        "Parameter %s=%s is not appropriate for inout %s"
                        % (p, input_params[p], input_source.id)
                    )

            exec(
                'self.input_sources["%s"] = self.sim.NoisyCurrentSource(**pynn_input_params)'
                % (input_source.id)
            )
        else:
            exec(
                'self.input_sources["%s"] = self.sim.%s(**input_params)'
                % (input_source.id, input_source.pynn_input)
            )

        # print(['%s (%s): %s'%(i, type(self.input_sources[i]),self.input_sources[i].simple_parameters()) for i in self.input_sources])

    def handle_document_start(self, id, notes):

        print_v("Document: %s" % id)

    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        if temperature:
            print_v("  Temperature: " + temperature)
        if notes:
            print_v("  Notes: " + notes)

    def handle_population(
        self, population_id, component, size=-1, component_obj=None, properties={}
    ):

        sizeInfo = " as yet unspecified size"
        if size >= 0:
            sizeInfo = ", size: " + str(size) + " cells"
        if component_obj:
            compInfo = " (%s)" % component_obj.__class__.__name__
        else:
            compInfo = ""

        print_v(
            "Population: "
            + population_id
            + ", component: "
            + component
            + compInfo
            + sizeInfo
        )

        exec(
            'self.POP_%s = self.sim.Population(%s, self.cells["%s"], label="%s")'
            % (population_id, size, component, population_id)
        )
        # exec('print_v(self.POP_%s)'%(population_id))
        exec('self.populations["%s"] = self.POP_%s' % (population_id, population_id))

    #
    #  Should be overridden to create specific cell instance
    #
    def handle_location(self, id, population_id, component, x, y, z):
        # self.printLocationInformation(id, population_id, component, x, y, z)

        exec("self.POP_%s.positions[0][%s] = %s" % (population_id, id, x))
        exec("self.POP_%s.positions[1][%s] = %s" % (population_id, id, y))
        exec("self.POP_%s.positions[2][%s] = %s" % (population_id, id, z))

    def handle_projection(
        self,
        projName,
        prePop,
        postPop,
        synapse,
        hasWeights=False,
        hasDelays=False,
        type="projection",
        synapse_obj=None,
        pre_synapse_obj=None,
    ):

        synInfo = ""
        if synapse_obj:
            synInfo += " (syn: %s)" % synapse_obj.__class__.__name__

        if pre_synapse_obj:
            synInfo += " (pre comp: %s)" % pre_synapse_obj.__class__.__name__

        print_v(
            "Projection: "
            + projName
            + " ("
            + type
            + ") from "
            + prePop
            + " to "
            + postPop
            + " with syn: "
            + synapse
            + synInfo
        )

        exec("self.projection__%s_conns = []" % (projName))

    #
    #  Should be overridden to handle network connection
    #
    def handle_connection(
        self,
        projName,
        id,
        prePop,
        postPop,
        synapseType,
        preCellId,
        postCellId,
        preSegId=0,
        preFract=0.5,
        postSegId=0,
        postFract=0.5,
        delay=0,
        weight=1,
    ):

        # self.print_connection_information(projName, id, prePop, postPop, synapseType, preCellId, postCellId, weight)
        # print_v("Src cell: %d, seg: %f, fract: %f -> Tgt cell %d, seg: %f, fract: %f; weight %s, delay: %s ms" % (preCellId,preSegId,preFract,postCellId,postSegId,postFract, weight, delay))

        exec(
            "self.projection__%s_conns.append((%s,%s,float(%s),float(%s)))"
            % (projName, preCellId, postCellId, weight, delay)
        )

    #
    #  Should be overridden to handle end of network connection
    #
    def finalise_projection(
        self, projName, prePop, postPop, synapse=None, type="projection"
    ):

        print_v(
            "Projection finalising: "
            + projName
            + " from "
            + prePop
            + " to "
            + postPop
            + " completed"
        )

        # exec('print(self.projection__%s_conns)'%projName)
        exec(
            'self.projection__%s_connector = self.sim.FromListConnector(self.projection__%s_conns, column_names=["weight", "delay"])'
            % (projName, projName)
        )

        exec(
            'self.projections["%s"] = self.sim.Projection(self.populations["%s"],self.populations["%s"], '
            % (projName, prePop, postPop)
            + "connector=self.projection__%s_connector, " % projName
            + "synapse_type=self.sim.StaticSynapse(weight=%s, delay=%s), " % (1, 5)
            + 'receptor_type="%s", ' % (self.receptor_types[synapse])
            + 'label="%s")' % projName
        )

        # exec('print(self.projections["%s"].describe())'%projName)

    #
    #  Should be overridden to create input source array
    #
    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):

        self.print_input_information(inputListId, population_id, component, size)

        if size < 0:
            self.log.error(
                "Error! Need a size attribute in sites element to create spike source!"
            )
            return

        self.input_info[inputListId] = (population_id, component)

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):

        # print_v("Input: %s[%s], cellId: %i, seg: %i, fract: %f, weight: %f" % (inputListId,id,cellId,segId,fract,weight))

        population_id, component = self.input_info[inputListId]

        # exec('print  self.POP_%s'%(population_id))
        # exec('print  self.POP_%s[%s]'%(population_id,cellId))

        exec(
            "self.POP_%s[%s].inject(self.input_sources[component]) "
            % (population_id, cellId)
        )
        # exec('self.input_sources[component].inject_into(self.populations["%s"])'%(population_id))

        # exec('pulse = self.sim.DCSource(amplitude=0.9, start=19, stop=89)')
        # pulse.inject_into(pop_pre)
        # exec('self.populations["pop0"][0].inject(pulse)')

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def finalise_input_source(self, inputName):
        print_v("Input : %s completed" % inputName)



#
#
#   A class to write to the MDF format (https://github.com/ModECI/MDF)...
#
#

from neuromllite.DefaultNetworkHandler import DefaultNetworkHandler
from neuromllite.utils import print_v
from modelspec.utils import save_to_json_file
from modelspec.utils import save_to_yaml_file
from modelspec.utils import locate_file
from modelspec.utils import evaluate
from pyneuroml.pynml import get_value_in_si

import lems.api as lems  # from pylems


class MDFHandler(DefaultNetworkHandler):
    def __init__(self, nl_network):
        print_v("Initiating PsyNeuLink handler")
        self.nl_network = nl_network

    def handle_document_start(self, id, notes):

        print_v("Parsing for PsyNeuLink export: %s" % id)
        self.id = id

        self.mdf_info = {}

        # from modeci_mdf import MODECI_MDF_VERSION
        MODECI_MDF_VERSION = "0.3"

        self.mdf_info[self.id] = {}
        self.mdf_info[self.id]["format"] = "ModECI MDF v%s" % MODECI_MDF_VERSION
        self.mdf_info[self.id]["graphs"] = {}

        self.pnl_additions = False

    def finalise_document(self):

        print_v("Writing file for...: %s" % self.id)

        save_to_json_file(self.mdf_info, "%s.mdf.json" % self.id, indent=4)
        save_to_yaml_file(self.mdf_info, "%s.mdf.yaml" % self.id, indent=4)
        """save_to_json_file(self.mdf_info_hl, '%s.bids-mdf.highlevel.json'%self.id, indent=4)"""

    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        self.network_id = network_id

        if temperature:
            print_v("  Temperature: " + temperature)
        if notes:
            print_v("  Notes: " + notes)

        self.mdf_graph = {}
        self.mdf_graph["notes"] = notes
        self.mdf_graph["nodes"] = {}
        self.mdf_graph["edges"] = {}

        self.mdf_info[self.id]["graphs"][network_id] = self.mdf_graph

    def _convert_value(self, val):

        funcs = ["exp"]
        for f in funcs:
            if "%s(" % f in val:
                val = val.replace("%s(" % f, "math.%s(" % f)

        val = evaluate(val)  # catch if it's an int etc.
        return val

    def handle_population(
        self, population_id, component, size=-1, component_obj=None, properties={}
    ):

        sizeInfo = " as yet unspecified size"
        if size >= 0:
            sizeInfo = ", size: " + str(size) + " cells"
        if component_obj:
            compInfo = " (%s)" % component_obj.__class__.__name__
        else:
            compInfo = ""

        print_v(
            "Population: "
            + population_id
            + ", component: "
            + component
            + compInfo
            + sizeInfo
            + ", properties: %s"% properties
        )

        if size >= 0:
            for i in range(size):
                node_id = "%s_%i" % (population_id, i)
                node = {}

                comp = self.nl_network.get_child(component, "cells")
                base_dir = "./"  # for now...

                if properties:
                    node["metadata"] = properties

                node["parameters"] = {}
                node["input_ports"] = {}
                node["output_ports"] = {}
                if comp is not None:
                    if comp.lems_source_file:
                        fname = locate_file(comp.lems_source_file, base_dir)
                        model = MDFHandler._load_lems_file_with_neuroml2_types(fname)
                        lems_comp = model.components.get(component)

                    if comp.neuroml2_cell:
                        model = MDFHandler._get_lems_model_with_neuroml2_types()
                        lems_comp = lems.Component(
                            id_=comp.id, type_=comp.neuroml2_cell
                        )
                        for p in comp.parameters:
                            lems_comp.set_parameter(
                                p,
                                evaluate(
                                    comp.parameters[p], self.nl_network.parameters
                                ),
                            )

                    print_v("All LEMS comps in model: %s" % model.components.keys())
                    print_v("This comp: %s" % lems_comp)
                    comp_type_name = lems_comp.type
                    lems_comp_type = model.component_types.get(comp_type_name)
                    notes = "Cell: [%s] is defined in %s and in Lems is: %s" % (
                        comp,
                        comp.lems_source_file,
                        lems_comp,
                    )

                    node["notes"] = notes

                    for p in lems_comp.parameters:
                        node["parameters"][p] = {
                            "value": get_value_in_si(evaluate(lems_comp.parameters[p]))
                        }

                    for c in lems_comp_type.constants:
                        node["parameters"][c.name] = {"value": get_value_in_si(c.value)}

                    for sv in lems_comp_type.dynamics.state_variables:
                        node["parameters"][sv.name] = {}
                        if sv.exposure:
                            node["output_ports"][sv.exposure] = {"value": sv.name}

                    for dv in lems_comp_type.dynamics.derived_variables:

                        print_v(
                            "Converting: %s (exp: %s) = [%s] or [%s]"
                            % (dv.name, dv.exposure, dv.value, dv.select)
                        )
                        if dv.name == "INPUT":
                            node["input_ports"][dv.name] = {}
                        else:
                            if dv.value is not None:
                                node["parameters"][dv.name] = {
                                    "value": self._convert_value(dv.value)
                                }
                                if dv.exposure:
                                    node["output_ports"][dv.exposure] = {
                                        "value": dv.name
                                    }
                            if dv.select is not None:
                                in_port = dv.select.replace("[*]/", "_")
                                node["input_ports"][in_port] = {}
                                node["parameters"][dv.name] = {"value": in_port}

                    conditions = 0
                    for eh in lems_comp_type.dynamics.event_handlers:

                        print_v("Converting: %s (type: %s)" % (eh, type(eh)))
                        if type(eh) == lems.OnStart:
                            for a in eh.actions:
                                if type(a) == lems.StateAssignment:
                                    node["parameters"][a.variable][
                                        "default_initial_value"
                                    ] = a.value
                        if type(eh) == lems.OnCondition:
                            test = (
                                eh.test.replace(".gt.", ">")
                                .replace(".geq.", ">=")
                                .replace(".lt.", "<")
                                .replace(".leq.", "<=")
                                .replace(".eq.", "==")
                            )
                            for a in eh.actions:
                                if type(a) == lems.StateAssignment:
                                    if (
                                        not "conditions"
                                        in node["parameters"][a.variable]
                                    ):
                                        node["parameters"][a.variable][
                                            "conditions"
                                        ] = {}

                                    node["parameters"][a.variable]["conditions"][
                                        "condition_%i" % conditions
                                    ] = {"test": test, "value": a.value}
                            conditions += 1

                    for td in lems_comp_type.dynamics.time_derivatives:
                        node["parameters"][td.variable][
                            "time_derivative"
                        ] = self._convert_value(td.value)

                self.mdf_graph["nodes"][node_id] = node

    # TODO: move to pylems!
    @classmethod
    def _get_all_children_in_lems(cls, component_type, model, child_type):
        c = []
        if child_type == "exposure":
            for e in component_type.exposures:
                c.append(e)

        if component_type.extends:
            ect = model.component_types[component_type.extends]
            c.extend(cls._get_all_children_in_lems(ect, model, child_type))

        return c

    # TODO: move to pyneuroml!
    @classmethod
    def _get_lems_model_with_neuroml2_types(cls):

        from pyneuroml.pynml import get_path_to_jnml_jar
        from pyneuroml.pynml import read_lems_file
        from lems.parser.LEMS import LEMSFileParser
        import zipfile

        lems_model = lems.Model(include_includes=False)
        parser = LEMSFileParser(lems_model)

        jar_path = get_path_to_jnml_jar()
        # print_comment_v("Loading standard NeuroML2 dimension/unit definitions from %s"%jar_path)
        jar = zipfile.ZipFile(jar_path, "r")
        new_lems = jar.read("NeuroML2CoreTypes/NeuroMLCoreDimensions.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/NeuroMLCoreCompTypes.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/Cells.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/Networks.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/Simulation.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/Synapses.xml")
        parser.parse(new_lems)
        new_lems = jar.read("NeuroML2CoreTypes/PyNN.xml")
        parser.parse(new_lems)

        return lems_model

    # TODO: move to pyneuroml!
    @classmethod
    def _load_lems_file_with_neuroml2_types(cls, lems_filename):

        from pyneuroml.pynml import read_lems_file

        lems_model = cls._get_lems_model_with_neuroml2_types()

        model = read_lems_file(
            lems_filename,
            include_includes=False,
            fail_on_missing_includes=True,
            debug=True,
        )

        for cid, c in model.components.items():
            lems_model.components[cid] = c
        for ctid, ct in model.component_types.items():
            lems_model.component_types[ctid] = ct

        return lems_model

    def handle_location(self, id, population_id, component, x, y, z):
        pass

    def finalise_population(self, population_id):

        pass

    #
    #  Should be overridden to handle network connection
    #
    def handle_connection(
        self,
        projName,
        id,
        prePop,
        postPop,
        synapseType,
        preCellId,
        postCellId,
        preSegId=0,
        preFract=0.5,
        postSegId=0,
        postFract=0.5,
        delay=0,
        weight=1,
    ):

        # self.print_connection_information(projName, id, prePop, postPop, synapseType, preCellId, postCellId, weight)
        print_v(
            ">>>>>> Src cell: %d, seg: %f, fract: %f -> Tgt cell %d, seg: %f, fract: %f; weight %s, delay: %s ms"
            % (
                preCellId,
                preSegId,
                preFract,
                postCellId,
                postSegId,
                postFract,
                weight,
                delay,
            )
        )

        pre_node_id = "%s_%i" % (prePop, preCellId)
        post_node_id = "%s_%i" % (postPop, postCellId)
        edge_id = "Edge %s to %s" % (pre_node_id, post_node_id)
        edge = {}
        edge["name"] = edge_id
        # edge['type'] = {}
        # edge['type']['NeuroML'] = synapseType
        # edge['parameters'] = {}
        # edge['functions'] = {}
        edge["sender_port"] = "OUTPUT"
        edge["receiver_port"] = "INPUT"
        edge["sender"] = pre_node_id
        edge["receiver"] = post_node_id
        edge["weight"] = weight

        """edge['type'] =  {
                        "PNL": "MappingProjection",
                        "generic": None
                    }"""

        self.mdf_graph["edges"][edge_id] = edge

    #
    #  Should be overridden to create input source array
    #
    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):

        pass

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):

        print_v(
            "Input: %s[%s], cellId: %i, seg: %i, fract: %f, weight: %f"
            % (inputListId, id, cellId, segId, fract, weight)
        )

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def finalise_input_source(self, inputName):
        print_v("Input : %s completed" % inputName)


if __name__ == "__main__":

    lems_model = MDFHandler._load_lems_file_with_neuroml2_types(
        "../../git/MDFTests/NeuroML/LEMS_SimABCD.xml"
    )

    print(
        "Loaded LEMS with\n > Dims: %s\n\n > CompTypes: %s\n\n > Comps: %s"
        % (
            lems_model.dimensions.keys(),
            lems_model.component_types.keys(),
            lems_model.components.keys(),
        )
    )



from neuromllite import Network, Cell, InputSource, Population, Synapse
from neuromllite import Projection, RandomConnectivity, Input, Simulation
import sys

################################################################################
###   Build new network

net = Network(id="Example8_Extension")
net.notes = "Example 8: general testing..."

net.seed = 7890
net.temperature = 32.0

net.parameters = {"N": 10, "fractionE": 0.8, "weightInput": 1}

cell = Cell(id="hhcell", neuroml2_source_file="test_files/hhcell.cell.nml")
net.cells.append(cell)


input_source = InputSource(
    id="poissonFiringSyn", neuroml2_source_file="test_files/inputs.nml"
)
net.input_sources.append(input_source)
"""
input_source = InputSource(id='iclamp0',
                           pynn_input='DCSource',
                           parameters={'amplitude':0.2, 'start':100., 'stop':900.})"""

net.input_sources.append(input_source)


pE = Population(
    id="Epop",
    size="int(N*fractionE)",
    component=cell.id,
    properties={"color": ".7 0 0"},
)
pRS = Population(
    id="RSpop",
    size="N - int(N*fractionE)",
    component=cell.id,
    properties={"color": "0 0 .7"},
)

net.populations.append(pE)
net.populations.append(pRS)

net.synapses.append(
    Synapse(id="ampa", neuroml2_source_file="test_files/ampa.synapse.nml")
)
net.synapses.append(
    Synapse(id="gaba", neuroml2_source_file="test_files/gaba.synapse.nml")
)


net.projections.append(
    Projection(
        id="projEI",
        presynaptic=pE.id,
        postsynaptic=pRS.id,
        synapse="ampa",
        delay=2,
        weight=0.2,
        random_connectivity=RandomConnectivity(probability=0.8),
    )
)


net.inputs.append(
    Input(
        id="stim",
        input_source=input_source.id,
        population=pE.id,
        percentage=50,
        weight="weightInput",
    )
)

print(net)
print(net.to_json())
new_file = net.to_json_file("%s.json" % net.id)


################################################################################
###   Build Simulation object & save as JSON

sim = Simulation(
    id="SimExample8",
    network=new_file,
    duration="1000",
    seed="1111",
    dt="0.025",
    record_traces={"all": "*"},
)

sim.to_json_file()


################################################################################
###   Run in some simulators

from neuromllite.NetworkGenerator import check_to_generate_or_run
import sys

check_to_generate_or_run(sys.argv, sim)



from pyNN.serialization import import_from_sonata, load_sonata_simulation_plan
import pyNN.nest as sim

simulation_plan = load_sonata_simulation_plan("simulation_config.json")
simulation_plan.setup(sim)
net = import_from_sonata("circuit_config.json", sim)

simulation_plan.execute(net)



#
#
#   A class to handle events from the NeuroMLHdf5Parser, etc.
#   This should be overridden by simulator specific implementations.
#   Parsing classes, e.g. NetworkBuilder should call the appropriate
#   function here when a cell location, connection, etc. is encountered.
#
#   Use of this handler class should mean that the network setup is
#   independent of the source of the network info (XML or HDF5 based NeuroML
#   files for example) and the instantiator of the network objects (NetManagerNEURON
#   or PyNN based setup class)
#
#   Author: Padraig Gleeson
#
#   This file has been developed as part of the neuroConstruct project
#   This work has been funded by the Medical Research Council & Wellcome Trust
#
#

from neuromllite.utils import print_v


class DefaultNetworkHandler:

    #
    #  Internal info method, can be reused in overriding classes for debugging
    #
    def print_location_information(self, id, population_id, component, x, y, z):
        position = "(%s, %s, %s)" % (x, y, z)
        print_v(
            "  Location "
            + str(id)
            + " of population: "
            + population_id
            + ", component: "
            + component
            + ": "
            + position
        )

    #
    #  Internal info method, can be reused in overriding classes for debugging
    #
    def print_connection_information(
        self, projName, id, prePop, postPop, synapseType, preCellId, postCellId, weight
    ):
        print_v(
            "  Connection "
            + str(id)
            + " of: "
            + projName
            + ": cell "
            + str(preCellId)
            + " in "
            + prePop
            + " -> cell "
            + str(postCellId)
            + " in "
            + postPop
            + ", syn: "
            + str(synapseType)
            + ", weight: "
            + str(weight)
        )

    #
    #  Internal info method, can be reused in overriding classes for debugging
    #
    def print_input_information(self, inputName, population_id, component, size=-1):
        sizeInfo = " size: " + str(size) + " cells"
        print_v(
            "Input Source: "
            + inputName
            + ", on population: "
            + population_id
            + sizeInfo
            + " with component: "
            + component
        )

    #
    #  Should be overridden
    #
    def handle_document_start(self, id, notes):

        print_v("Document: %s started..." % id)
        if notes:
            print_v("  Notes: " + notes)

    #
    #  Should be overridden
    #
    def finalise_document(self):

        print_v("Document ended...")

    #
    #  Should be overridden to create network
    #
    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        if temperature:
            print_v("  Temperature: " + temperature)
        if notes:
            print_v("  Notes: " + notes)

    #
    #  Should be overridden to create population array
    #
    def handle_population(
        self, population_id, component, size=-1, component_obj=None, properties={}
    ):
        sizeInfo = " as yet unspecified size"
        if size >= 0:
            sizeInfo = " size: " + str(size) + " cells"
        if component_obj:
            compInfo = " (%s)" % component_obj.__class__.__name__
        else:
            compInfo = ""
        propsInfo = ""
        if properties and len(properties) > 0:
            propsInfo += "; %s" % properties

        print_v(
            "Population: "
            + population_id
            + ", component: "
            + component
            + compInfo
            + sizeInfo
            + propsInfo
        )

    #
    #  Should be overridden to create specific cell instance
    #
    def handle_location(self, id, population_id, component, x, y, z):
        self.print_location_information(id, population_id, component, x, y, z)

    def finalise_population(self, population_id):

        pass

    #
    #  Should be overridden to create population array
    #
    def handle_projection(
        self,
        projName,
        prePop,
        postPop,
        synapse,
        hasWeights=False,
        hasDelays=False,
        type="projection",
        synapse_obj=None,
        pre_synapse_obj=None,
    ):

        synInfo = ""
        if synapse_obj:
            synInfo += " (syn: %s)" % synapse_obj.__class__.__name__

        if pre_synapse_obj:
            synInfo += " (pre comp: %s)" % pre_synapse_obj.__class__.__name__

        print_v(
            "Projection: "
            + projName
            + " ("
            + type
            + ") from "
            + prePop
            + " to "
            + postPop
            + " with syn: "
            + synapse
            + synInfo
        )

    #
    #  Should be overridden to handle network connection
    #
    def handle_connection(
        self,
        projName,
        id,
        prePop,
        postPop,
        synapseType,
        preCellId,
        postCellId,
        preSegId=0,
        preFract=0.5,
        postSegId=0,
        postFract=0.5,
        delay=0,
        weight=1,
    ):

        self.print_connection_information(
            projName, id, prePop, postPop, synapseType, preCellId, postCellId, weight
        )
        if preSegId != 0 or postSegId != 0 or preFract != 0.5 or postFract != 0.5:
            print_v(
                "Src cell: %d, seg: %f, fract: %f -> Tgt cell %d, seg: %f, fract: %f; weight %s, delay: %s ms"
                % (
                    preCellId,
                    preSegId,
                    preFract,
                    postCellId,
                    postSegId,
                    postFract,
                    weight,
                    delay,
                )
            )

    #
    #  Should be overridden to handle end of network connection
    #
    def finalise_projection(
        self, projName, prePop, postPop, synapse=None, type="projection"
    ):

        print_v(
            "Projection: "
            + projName
            + " from "
            + prePop
            + " to "
            + postPop
            + " completed"
        )

    #
    #  Should be overridden to create input source array
    #
    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):

        self.print_input_information(inputListId, population_id, component, size)

        if size < 0:
            self.log.error(
                "Error! Need a size attribute in sites element to create spike source!"
            )
            return

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):

        print_v(
            "  Input: %s[%s], cellId: %i, seg: %i, fract: %f, weight: %f"
            % (inputListId, id, cellId, segId, fract, weight)
        )

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def finalise_input_source(self, inputName):
        print_v("Input : %s completed" % inputName)



from neuromllite import Network, Cell, InputSource, Population, Synapse
from neuromllite import Projection, RandomConnectivity, Input, Simulation
import sys

################################################################################
###   Build new network

net = Network(id="Example9_HindmarshRose")
net.notes = "Example 9: HindmarshRose"
net.parameters = {"N": 1}

cell = Cell(id="hrCell", lems_source_file="test_files/HindmarshRose3d.xml")
cell.parameters = {}

params = {
    "a": 1,
    "b": 3,
    "c": -3.0,
    "d": 5.0,
    "s": 4.0,
    "I": 5.0,
    "x1": -1.3,
    "r": 0.002,
    "x0": -1.3,
    "y0": -1.0,
    "z0": 1.0,
}

for p in params:
    cell.parameters[p] = p
    net.parameters[p] = params[p]

net.cells.append(cell)


pop = Population(
    id="hrPop", size="1", component=cell.id, properties={"color": ".7 0 0"}
)
net.populations.append(pop)


print(net)
print(net.to_json())
new_file = net.to_json_file("%s.json" % net.id)


################################################################################
###   Build Simulation object & save as JSON

sim = Simulation(
    id="SimExample9",
    network=new_file,
    duration="1000000",
    dt="25",
    record_variables={"x": {"all": "*"}, "y": {"all": "*"}, "z": {"all": "*"}},
    plots2D={
        "X-Y": {"x_axis": "hrPop[0]/x", "y_axis": "hrPop[0]/y"},
        "Y-Z": {"x_axis": "hrPop[0]/y", "y_axis": "hrPop[0]/z"},
        "X-Z": {"x_axis": "hrPop[0]/x", "y_axis": "hrPop[0]/z"},
    },
    plots3D={
        "X-Y-Z": {
            "x_axis": "hrPop[0]/x",
            "y_axis": "hrPop[0]/y",
            "z_axis": "hrPop[0]/z",
        }
    },
)

sim.to_json_file()


################################################################################
###   Run in some simulators

from neuromllite.NetworkGenerator import check_to_generate_or_run
import sys

check_to_generate_or_run(sys.argv, sim)



from neuromllite import (
    Network,
    Cell,
    InputSource,
    Population,
    Synapse,
    RectangularRegion,
    RandomLayout,
)
from neuromllite import Projection, RandomConnectivity, Input, Simulation
import sys

################################################################################
###   Build new network

net = Network(id="SonataExample")
net.notes = "Example for testing Sonata"
net.parameters = {"input_amp": 0.190, "input_del": 100, "input_dur": 800}

cell = Cell(id="testcell", pynn_cell="IF_cond_alpha")
cell.parameters = {
    "i_offset": 0,
    "cm": 0.117,
    "tau_m": 22.1,
    "tau_refrac": 3,
    "v_reset": -50,
    "v_rest": -78,
    "v_thresh": -47,
}


net.cells.append(cell)
"""
cell2 = Cell(id='testcell2', pynn_cell='IF_cond_alpha')
cell2.parameters = { "tau_refrac":5, "i_offset":-.1 }
net.cells.append(cell2)"""

input_source = InputSource(
    id="i_clamp",
    pynn_input="DCSource",
    parameters={
        "amplitude": "input_amp",
        "start": "input_del",
        "stop": "input_del+input_dur",
    },
)
net.input_sources.append(input_source)

r1 = RectangularRegion(id="region1", x=0, y=0, z=0, width=1000, height=100, depth=1000)
net.regions.append(r1)

p0 = Population(
    id="pop0",
    size=2,
    component=cell.id,
    properties={"color": "1 0 0"},
    random_layout=RandomLayout(region=r1.id),
)
net.populations.append(p0)
"""
p1 = Population(id='pop1', size=2, component=cell2.id, properties={'color':'0 1 0'},random_layout = RandomLayout(region=r1.id))
p2 = Population(id='pop2', size=1, component=cell2.id, properties={'color':'0 0 1'},random_layout = RandomLayout(region=r1.id))

net.populations.append(p1)
net.populations.append(p2)"""

"""
net.synapses.append(Synapse(id='ampaSyn', 
                            pynn_receptor_type='excitatory', 
                            pynn_synapse_type='cond_alpha', 
                            parameters={'e_rev':-10, 'tau_syn':2}))
net.synapses.append(Synapse(id='gabaSyn', 
                            pynn_receptor_type='inhibitory', 
                            pynn_synapse_type='cond_alpha', 
                            parameters={'e_rev':-80, 'tau_syn':10}))

net.projections.append(Projection(id='proj0',
                                  presynaptic=p0.id, 
                                  postsynaptic=p1.id,
                                  synapse='ampaSyn',
                                  delay=2,
                                  weight=0.02))
net.projections[0].random_connectivity=RandomConnectivity(probability=1)

net.projections.append(Projection(id='proj1',
                                  presynaptic=p0.id, 
                                  postsynaptic=p2.id,
                                  synapse='gabaSyn',
                                  delay=2,
                                  weight=0.01))
net.projections[1].random_connectivity=RandomConnectivity(probability=1)"""

net.inputs.append(
    Input(id="stim", input_source=input_source.id, population=p0.id, percentage=50)
)

print(net.to_json())
new_file = net.to_json_file("%s.json" % net.id)


################################################################################
###   Build Simulation object & save as JSON

sim = Simulation(
    id="SimSonataExample",
    network=new_file,
    duration="1000",
    dt="0.01",
    recordTraces={"all": "*"},
    recordSpikes={"pop0": "*"},
)

sim.to_json_file()


################################################################################
###   Run in some simulators

from neuromllite.NetworkGenerator import check_to_generate_or_run
import sys

check_to_generate_or_run(sys.argv, sim)



import os, sys

os.chdir("..")
sys.path.append(".")

print("Running PyNN Ex4 test script with %s" % sys.argv)

if "nest" in sys.argv:
    sys.argv.append("-pynnnest")
if "neuron" in sys.argv:
    sys.argv.append("-pynnnrn")
if "brian" in sys.argv:
    sys.argv.append("-pynnbrian")

import Example4



#
#
#   A class to build networks in BindsNET (work in progress)...
#
#

from neuromllite.utils import print_v
from neuromllite.utils import evaluate
from neuromllite.DefaultNetworkHandler import DefaultNetworkHandler

from pyneuroml.pynml import convert_to_units

import numpy as np

import bindsnet


class BindsNETHandler(DefaultNetworkHandler):

    pops_vs_components = {}
    pops_vs_bn_layers = {}

    proj_weights = {}
    proj_delays = {}

    input_info = {}

    """
    populations = {}
    projections = {}
    input_sources = {}

    inputs = []
    cells = {}"""

    pop_indices_vs_gids = {}

    def __init__(self, nl_network):
        print_v("Initiating BindsNET...")
        self.nl_network = nl_network
        self.curr_gid = 0

        self.bn_network = bindsnet.network.Network(dt=1.0)

    def handle_document_start(self, id, notes):
        print_v("Document: %s" % id)

    def handle_network(self, network_id, notes, temperature=None):

        print_v("Network: %s" % network_id)
        if temperature:
            print_v("  Temperature: " + temperature)
        if notes:
            print_v("  Notes: " + notes)

    def handle_population(
        self, population_id, component, size=-1, component_obj=None, properties={}
    ):

        sizeInfo = " as yet unspecified size"
        if size >= 0:
            sizeInfo = ", size: " + str(size) + " cells"
        if component_obj:
            compInfo = " (%s)" % component_obj.__class__.__name__
        else:
            compInfo = ""

        print_v(
            "Population: "
            + population_id
            + ", component: "
            + component
            + compInfo
            + sizeInfo
        )

        self.pops_vs_components[population_id] = component
        self.pop_indices_vs_gids[population_id] = {}

    #
    #  Should be overridden to create specific cell instance
    #
    def handle_location(self, id, population_id, component, x, y, z):
        # self.printLocationInformation(id, population_id, component, x, y, z)

        self.pop_indices_vs_gids[population_id][id] = self.curr_gid
        self.curr_gid += 1

        """
        exec('self.POP_%s.positions[0][%s] = %s'%(population_id,id,x))
        exec('self.POP_%s.positions[1][%s] = %s'%(population_id,id,y))
        exec('self.POP_%s.positions[2][%s] = %s'%(population_id,id,z))"""

    def handle_projection(
        self,
        projName,
        prePop,
        postPop,
        synapse,
        hasWeights=False,
        hasDelays=False,
        type="projection",
        synapse_obj=None,
        pre_synapse_obj=None,
    ):

        synInfo = ""
        if synapse_obj:
            synInfo += " (syn: %s)" % synapse_obj.__class__.__name__

        if pre_synapse_obj:
            synInfo += " (pre comp: %s)" % pre_synapse_obj.__class__.__name__

        print_v(
            "Projection: "
            + projName
            + " ("
            + type
            + ") from "
            + prePop
            + " to "
            + postPop
            + " with syn: "
            + synapse
            + synInfo
        )

        self.proj_weights[projName] = np.zeros(
            (
                len(self.pop_indices_vs_gids[prePop]),
                len(self.pop_indices_vs_gids[postPop]),
            )
        )
        self.proj_delays[projName] = np.zeros(
            (
                len(self.pop_indices_vs_gids[prePop]),
                len(self.pop_indices_vs_gids[postPop]),
            )
        )

        """
        exec('self.projection__%s_conns = []'%(projName))"""

    #
    #  Should be overridden to handle network connection
    #
    def handle_connection(
        self,
        projName,
        id,
        prePop,
        postPop,
        synapseType,
        preCellId,
        postCellId,
        preSegId=0,
        preFract=0.5,
        postSegId=0,
        postFract=0.5,
        delay=0,
        weight=1,
    ):

        self.print_connection_information(
            projName, id, prePop, postPop, synapseType, preCellId, postCellId, weight
        )
        print_v(
            "Src cell: %d, seg: %f, fract: %f -> Tgt cell %d, seg: %f, fract: %f; weight %s, delay: %s ms"
            % (
                preCellId,
                preSegId,
                preFract,
                postCellId,
                postSegId,
                postFract,
                weight,
                delay,
            )
        )

        self.proj_weights[projName][preCellId][postCellId] = weight
        self.proj_delays[projName][preCellId][postCellId] = delay

    #
    #  Should be overridden to handle end of network connection
    #
    def finalise_projection(
        self, projName, prePop, postPop, synapse=None, type="projection"
    ):

        print_v(
            "Projection finalising: "
            + projName
            + " from "
            + prePop
            + " to "
            + postPop
            + " completed"
        )

    #
    #  Should be overridden to create input source array
    #
    def handle_input_list(
        self, inputListId, population_id, component, size, input_comp_obj=None
    ):

        self.print_input_information(inputListId, population_id, component, size)
        if input_comp_obj:
            print("Input comp: %s" % input_comp_obj)

        if size < 0:
            self.log.error(
                "Error! Need a size attribute in sites element to create spike source!"
            )
            return

        self.input_info[inputListId] = (population_id, component)

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def handle_single_input(
        self, inputListId, id, cellId, segId=0, fract=0.5, weight=1
    ):

        population_id, component = self.input_info[inputListId]

        print_v(
            "Input: %s[%s] (%s), pop: %s, cellId: %i, seg: %i, fract: %f, weight: %f"
            % (inputListId, id, component, population_id, cellId, segId, fract, weight)
        )

    #
    #  Should be overridden to to connect each input to the target cell
    #
    def finalise_input_source(self, inputName):
        print_v("Input: %s completed" % inputName)

    def finalise_document(self):
        print_v("Building network with: %s" % self.pop_indices_vs_gids)
        print_v("Weights: %s" % self.proj_weights)
        print_v("Delays: %s" % self.proj_delays)

        for pop in self.pop_indices_vs_gids:
            size = len(self.pop_indices_vs_gids[pop])
            comp = self.pops_vs_components[pop]
            cell = self.nl_network.get_child(comp, "cells")
            layer_name = "%s_bn_pop" % pop
            cmd = "self.pops_vs_bn_layers['%s'] = bindsnet.network.nodes.%s(%s)" % (
                pop,
                cell.bindsnet_node,
                size,
            )
            print(
                "Creating a population %s with %s instances of %s using: <%s>"
                % (pop, size, cell, cmd)
            )
            exec(cmd)



'''
NETPYNE simulator compliant export for:

Components:
    null (Type: notes)
    synInput (Type: expTwoSynapse:  tauRise=0.001 (SI time) tauDecay=0.005 (SI time) peakTime=0.002011797390542625 (SI time) waveformFactor=1.8691859765265255 (dimensionless) gbase=8.0E-9 (SI conductance) erev=0.02 (SI voltage))
    iclamp0 (Type: pulseGenerator:  delay=0.02 (SI time) duration=0.8 (SI time) amplitude=1.0000000000000002E-10 (SI current))
    spikeGenPoisson (Type: spikeGeneratorPoisson:  averageRate=50.0 (SI per_time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    poissonFiringSyn (Type: poissonFiringSynapse:  averageRate=10.0 (SI per_time) averageIsi=0.1 (SI time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    poissonFiringSyn100Hz (Type: poissonFiringSynapse:  averageRate=100.0 (SI per_time) averageIsi=0.01 (SI time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    passiveChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    naChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    kChan (Type: ionChannelHH:  conductance=1.0E-11 (SI conductance))
    hhcell (Type: cell)
    ampa (Type: expTwoSynapse:  tauRise=0.001 (SI time) tauDecay=0.002 (SI time) peakTime=0.0013862943611198907 (SI time) waveformFactor=4.0 (dimensionless) gbase=2.0E-9 (SI conductance) erev=0.0 (SI voltage))
    Example3_Network (Type: network)
    SimExample3 (Type: Simulation:  length=1.0 (SI time) step=2.5E-5 (SI time))


    This NETPYNE file has been generated by org.neuroml.export (see https://github.com/NeuroML/org.neuroml.export)
         org.neuroml.export  v1.8.1
         org.neuroml.model   v1.8.1
         jLEMS               v0.10.6

'''
# Main NetPyNE script for: Example3_Network

# See https://github.com/Neurosim-lab/netpyne

from netpyne import specs  # import netpyne specs module
from netpyne import sim    # import netpyne sim module
from netpyne import __version__ as version

from neuron import h

import sys
import time
import datetime

class NetPyNESimulation():

    def __init__(self, tstop, dt, seed=123456789, save_json=False):

        self.setup_start = time.time()
        
        self.report_file = open('report.SimExample3.txt','w')
        self.report_file.write('# Report of running simulation with NetPyNE v%s\n'%version)
        self.report_file.write('Simulator=NetPyNE\n')
        self.report_file.write('SimulatorVersion=%s\n'%version)
        self.report_file.write('SimulationFile=%s\n'%__file__)
        self.report_file.write('PythonVersion=%s\n'%sys.version.replace('\n',' '))
        self.report_file.write('NeuronVersion=%s\n'%h.nrnversion())
        self.report_file.write('NeuroMLExportVersion=1.8.1\n')
        

        ###############################################################################
        # NETWORK PARAMETERS
        ###############################################################################

        self.nml2_file_name = 'Example3_Network.net.nml'

        ###############################################################################
        # SIMULATION PARAMETERS
        ###############################################################################

        self.simConfig = specs.SimConfig()   # object of class SimConfig to store the simulation configuration

        # Simulation parameters
        self.simConfig.duration = self.simConfig.tstop = tstop # Duration of the simulation, in ms
        self.simConfig.dt = dt # Internal integration timestep to use

        # Seeds for randomizers (connectivity, input stimulation and cell locations)
        # Note: locations and connections should be fully specified by the structure of the NeuroML,
        # so seeds for conn & loc shouldn't affect networks structure/behaviour
        self.simConfig.seeds = {'conn': 0, 'stim': 5678, 'loc': 0} 

        self.simConfig.createNEURONObj = 1  # create HOC objects when instantiating network
        self.simConfig.createPyStruct = 1  # create Python structure (simulator-independent) when instantiating network
        self.simConfig.verbose = False  # show detailed messages 
        
        # Recording 
        self.simConfig.recordCells = ['all']  
        self.simConfig.recordTraces = {}
        self.simConfig.saveCellSecs=False
        self.simConfig.saveCellConns=False
        self.simConfig.gatherOnlySimData=True 

                # For saving to file: SimExample3.pop0.v.dat (ref: SimExample3_pop0_v_dat)
                                
        # Column: pop0_0_hhcell_v: Pop: pop0; cell: 0; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop0_v_dat_pop0_0_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop0','cellLabel':0}}
                                
        # Column: pop0_1_hhcell_v: Pop: pop0; cell: 1; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop0_v_dat_pop0_1_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop0','cellLabel':1}}
                                
        # Column: pop0_2_hhcell_v: Pop: pop0; cell: 2; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop0_v_dat_pop0_2_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop0','cellLabel':2}}
                                
        # Column: pop0_3_hhcell_v: Pop: pop0; cell: 3; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop0_v_dat_pop0_3_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop0','cellLabel':3}}
                                
        # Column: pop0_4_hhcell_v: Pop: pop0; cell: 4; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop0_v_dat_pop0_4_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop0','cellLabel':4}}
                        
                # For saving to file: SimExample3.pop1.v.dat (ref: SimExample3_pop1_v_dat)
                                
        # Column: pop1_0_hhcell_v: Pop: pop1; cell: 0; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_0_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':0}}
                                
        # Column: pop1_1_hhcell_v: Pop: pop1; cell: 1; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_1_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':1}}
                                
        # Column: pop1_2_hhcell_v: Pop: pop1; cell: 2; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_2_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':2}}
                                
        # Column: pop1_3_hhcell_v: Pop: pop1; cell: 3; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_3_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':3}}
                                
        # Column: pop1_4_hhcell_v: Pop: pop1; cell: 4; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_4_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':4}}
                                
        # Column: pop1_5_hhcell_v: Pop: pop1; cell: 5; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_5_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':5}}
                                
        # Column: pop1_6_hhcell_v: Pop: pop1; cell: 6; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_6_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':6}}
                                
        # Column: pop1_7_hhcell_v: Pop: pop1; cell: 7; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_7_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':7}}
                                
        # Column: pop1_8_hhcell_v: Pop: pop1; cell: 8; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_8_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':8}}
                                
        # Column: pop1_9_hhcell_v: Pop: pop1; cell: 9; segment id: 0; segment name: soma; Neuron loc: soma(0.5); value: v (v)
        self.simConfig.recordTraces['SimExample3_pop1_v_dat_pop1_9_soma_v'] = {'sec':'soma','loc':0.5,'var':'v','conds':{'pop':'pop1','cellLabel':9}}
                        
        
        self.simConfig.plotCells = ['all']

        self.simConfig.recordStim = True  # record spikes of cell stims
        self.simConfig.recordStep = self.simConfig.dt # Step size in ms to save data (eg. V traces, LFP, etc)

        # Analysis and plotting, see http://neurosimlab.org/netpyne/reference.html#analysis-related-functions
        self.simConfig.analysis['plotRaster'] = False  # Plot raster
        self.simConfig.analysis['plot2Dnet'] = False  # Plot 2D net cells and connections
        self.simConfig.analysis['plotSpikeHist'] = False # plot spike histogram
        self.simConfig.analysis['plotConn'] = False # plot network connectivity
        self.simConfig.analysis['plotSpikePSD'] = False # plot 3d architecture

        # Saving
        self.simConfig.filename = 'Example3_Network.txt'  # Set file output name
        self.simConfig.saveFileStep = self.simConfig.dt # step size in ms to save data to disk
        # self.simConfig.saveDat = True # save to dat file
        self.simConfig.saveJson = save_json # save to json file


    def run(self):

        ###############################################################################
        # IMPORT & RUN
        ###############################################################################

        print("Running a NetPyNE based simulation for %sms (dt: %sms) at %s degC"%(self.simConfig.duration, self.simConfig.dt, self.simConfig.hParams['celsius']))

        self.setup_sim_start = time.time()
        self.gids = sim.importNeuroML2SimulateAnalyze(self.nml2_file_name,self.simConfig)

        self.sim_end = time.time()
        self.setup_sim_time = self.sim_end - self.setup_sim_start
        print("Finished NetPyNE simulation in %f seconds (%f mins)..."%(self.setup_sim_time, self.setup_sim_time/60.0))

        try:
            self.save_results()
        except Exception as e:
            print("Exception saving results of NetPyNE simulation: %s" % (e))
            return


    def save_results(self):

        ###############################################################################
        #   Saving data (this ensures the data gets saved in the format/files 
        #   as specified in the LEMS <Simulation> element)
        ###############################################################################

        if sim.rank==0: 
        
            print("Saving traces to file: SimExample3.pop0.v.dat (ref: SimExample3_pop0_v_dat)")

                
            # Column: t
            col_SimExample3_pop0_v_dat_t = [i*self.simConfig.dt for i in range(int(self.simConfig.duration/self.simConfig.dt))]
                        
            # Column: pop0_0_hhcell_v: Pop: pop0; cell: 0; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop0_v_dat_pop0_0_hhcell_v = sim.allSimData['SimExample3_pop0_v_dat_pop0_0_soma_v']['cell_%s'%self.gids['pop0'][0]]
                        
            # Column: pop0_1_hhcell_v: Pop: pop0; cell: 1; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop0_v_dat_pop0_1_hhcell_v = sim.allSimData['SimExample3_pop0_v_dat_pop0_1_soma_v']['cell_%s'%self.gids['pop0'][1]]
                        
            # Column: pop0_2_hhcell_v: Pop: pop0; cell: 2; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop0_v_dat_pop0_2_hhcell_v = sim.allSimData['SimExample3_pop0_v_dat_pop0_2_soma_v']['cell_%s'%self.gids['pop0'][2]]
                        
            # Column: pop0_3_hhcell_v: Pop: pop0; cell: 3; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop0_v_dat_pop0_3_hhcell_v = sim.allSimData['SimExample3_pop0_v_dat_pop0_3_soma_v']['cell_%s'%self.gids['pop0'][3]]
                        
            # Column: pop0_4_hhcell_v: Pop: pop0; cell: 4; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop0_v_dat_pop0_4_hhcell_v = sim.allSimData['SimExample3_pop0_v_dat_pop0_4_soma_v']['cell_%s'%self.gids['pop0'][4]]
                
            dat_file_SimExample3_pop0_v_dat = open('SimExample3.pop0.v.dat', 'w')
            for i in range(len(col_SimExample3_pop0_v_dat_t)):
                dat_file_SimExample3_pop0_v_dat.write( '%s\t'%(col_SimExample3_pop0_v_dat_t[i]/1000.0) +  '%s\t'%(col_SimExample3_pop0_v_dat_pop0_0_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop0_v_dat_pop0_1_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop0_v_dat_pop0_2_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop0_v_dat_pop0_3_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop0_v_dat_pop0_4_hhcell_v[i]/1000.0) +  '\n')
            dat_file_SimExample3_pop0_v_dat.close()

        
            print("Saving traces to file: SimExample3.pop1.v.dat (ref: SimExample3_pop1_v_dat)")

                
            # Column: t
            col_SimExample3_pop1_v_dat_t = [i*self.simConfig.dt for i in range(int(self.simConfig.duration/self.simConfig.dt))]
                        
            # Column: pop1_0_hhcell_v: Pop: pop1; cell: 0; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_0_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_0_soma_v']['cell_%s'%self.gids['pop1'][0]]
                        
            # Column: pop1_1_hhcell_v: Pop: pop1; cell: 1; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_1_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_1_soma_v']['cell_%s'%self.gids['pop1'][1]]
                        
            # Column: pop1_2_hhcell_v: Pop: pop1; cell: 2; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_2_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_2_soma_v']['cell_%s'%self.gids['pop1'][2]]
                        
            # Column: pop1_3_hhcell_v: Pop: pop1; cell: 3; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_3_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_3_soma_v']['cell_%s'%self.gids['pop1'][3]]
                        
            # Column: pop1_4_hhcell_v: Pop: pop1; cell: 4; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_4_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_4_soma_v']['cell_%s'%self.gids['pop1'][4]]
                        
            # Column: pop1_5_hhcell_v: Pop: pop1; cell: 5; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_5_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_5_soma_v']['cell_%s'%self.gids['pop1'][5]]
                        
            # Column: pop1_6_hhcell_v: Pop: pop1; cell: 6; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_6_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_6_soma_v']['cell_%s'%self.gids['pop1'][6]]
                        
            # Column: pop1_7_hhcell_v: Pop: pop1; cell: 7; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_7_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_7_soma_v']['cell_%s'%self.gids['pop1'][7]]
                        
            # Column: pop1_8_hhcell_v: Pop: pop1; cell: 8; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_8_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_8_soma_v']['cell_%s'%self.gids['pop1'][8]]
                        
            # Column: pop1_9_hhcell_v: Pop: pop1; cell: 9; segment id: 0; segment name: soma; value: v
            col_SimExample3_pop1_v_dat_pop1_9_hhcell_v = sim.allSimData['SimExample3_pop1_v_dat_pop1_9_soma_v']['cell_%s'%self.gids['pop1'][9]]
                
            dat_file_SimExample3_pop1_v_dat = open('SimExample3.pop1.v.dat', 'w')
            for i in range(len(col_SimExample3_pop1_v_dat_t)):
                dat_file_SimExample3_pop1_v_dat.write( '%s\t'%(col_SimExample3_pop1_v_dat_t[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_0_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_1_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_2_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_3_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_4_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_5_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_6_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_7_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_8_hhcell_v[i]/1000.0) +  '%s\t'%(col_SimExample3_pop1_v_dat_pop1_9_hhcell_v[i]/1000.0) +  '\n')
            dat_file_SimExample3_pop1_v_dat.close()

        
        
            save_end = time.time()
            save_time = save_end - self.sim_end
            print("Finished saving results in %f seconds"%(save_time))


           
            self.report_file.write('StartTime=%s\n'%datetime.datetime.fromtimestamp(self.setup_start).strftime('%Y-%m-%d %H:%M:%S'))
            self.report_file.write('RealSetupAndSimulationTime=%s\n'%self.setup_sim_time)
            self.report_file.write('SimulationSaveTime=%s\n'%save_time)
            self.report_file.close()
            print("Saving report of simulation to %s"%('report.SimExample3.txt'))
        
if __name__ == '__main__':

    save_json = '-json' in sys.argv

    ns = NetPyNESimulation(tstop=1000.0, dt=0.025, seed=5678, save_json=save_json)

    ns.run()

    if '-nogui' in sys.argv:
        quit()



"""
Neuron simulator export for:

Components:
    spiker (Type: spikeGenerator:  period=0.012 (SI time))
    spikes2 (Type: spikeArray)
    spike100 (Type: spikeArray)
    if1 (Type: intFire1Cell:  tau=0.024 (SI time) C=1.0000000000000001E-11 (SI capacitance) thresh=0.001 (SI voltage) reset=0.0 (SI voltage) MVOLT=0.001 (SI voltage) PAMPERE=1.0E-12 (SI current))
    iaf1 (Type: iafCell:  leakConductance=5.0E-10 (SI conductance) leakReversal=-0.06 (SI voltage) thresh=-0.035 (SI voltage) reset=-0.065 (SI voltage) C=1.0000000000000001E-11 (SI capacitance))
    iaf2 (Type: iafCell:  leakConductance=5.2E-10 (SI conductance) leakReversal=-0.06 (SI voltage) thresh=-0.035 (SI voltage) reset=-0.07 (SI voltage) C=1.0000000000000001E-11 (SI capacitance))
    syn0 (Type: impulseSynapse:  PAMPERE=1.0E-12 (SI current) TSMALL=2.5E-6 (SI time))
    syn1 (Type: expOneSynapse:  tauDecay=0.002 (SI time) gbase=1.0000000000000002E-10 (SI conductance) erev=0.01 (SI voltage))
    syn2 (Type: expTwoSynapse:  tauRise=5.0E-5 (SI time) tauDecay=0.002 (SI time) peakTime=1.8917330533917622E-4 (SI time) waveformFactor=1.1273891364130213 (dimensionless) gbase=1.0000000000000002E-10 (SI conductance) erev=0.01 (SI voltage))
    net2 (Type: network)
    sim1 (Type: Simulation:  length=0.3 (SI time) step=5.0E-6 (SI time))


    This NEURON file has been generated by org.neuroml.export (see https://github.com/NeuroML/org.neuroml.export)
         org.neuroml.export  v1.5.4
         org.neuroml.model   v1.5.4
         jLEMS               v0.9.9.1

"""

import neuron

import time
import sys

import hashlib

h = neuron.h
h.load_file("nrngui.hoc")

h("objref p")
h("p = new PythonObject()")


class NeuronSimulation:
    def __init__(self, tstop, dt, seed=123456789):

        print(
            "\n    Starting simulation in NEURON of %sms generated from NeuroML2 model...\n"
            % tstop
        )

        self.seed = seed
        self.randoms = []
        self.next_global_id = (
            0  # Used in Random123 classes for elements using random(), etc.
        )

        self.next_spiking_input_id = (
            0  # Used in Random123 classes for elements using random(), etc.
        )

        """
        Adding simulation Component(id=sim1 type=Simulation) of network/component: net2 (Type: network)
        
        """
        # ######################   Population: spikersPop
        print(
            "Population spikersPop contains 1 instance(s) of component: spiker of type: spikeGenerator"
        )

        h(" {n_spikersPop = 1} ")
        """
        Population spikersPop contains instances of Component(id=spiker type=spikeGenerator)
        whose dynamics will be implemented as a mechanism (spiker) in a mod file
        """
        h(" create spikersPop[1]")
        h(" objectvar m_spiker_spikersPop[1] ")

        for i in range(int(h.n_spikersPop)):
            h.spikersPop[i].L = 10.0
            h.spikersPop[i](0.5).diam = 10.0
            h.spikersPop[i](0.5).cm = 318.31
            h.spikersPop[i].push()
            h(
                " spikersPop[%i]  { m_spiker_spikersPop[%i] = new spiker(0.5) } "
                % (i, i)
            )

            h.m_spiker_spikersPop[i].period = 12.0
            h.pop_section()

            self.next_global_id += 1

        # ######################   Population: spikeArrayPop
        print(
            "Population spikeArrayPop contains 1 instance(s) of component: spikes2 of type: spikeArray"
        )

        h(" {n_spikeArrayPop = 1} ")
        """
        Population spikeArrayPop contains instances of Component(id=spikes2 type=spikeArray)
        whose dynamics will be implemented as a mechanism (spikes2) in a mod file
        """
        h(" create spikeArrayPop[1]")
        h(" objectvar m_spikes2_spikeArrayPop[1] ")

        for i in range(int(h.n_spikeArrayPop)):
            h.spikeArrayPop[i].L = 10.0
            h.spikeArrayPop[i](0.5).diam = 10.0
            h.spikeArrayPop[i](0.5).cm = 318.31
            h.spikeArrayPop[i].push()
            h(
                " spikeArrayPop[%i]  { m_spikes2_spikeArrayPop[%i] = new spikes2(0.5) } "
                % (i, i)
            )

            h.pop_section()

            self.next_global_id += 1

        # ######################   Population: spike100Pop
        print(
            "Population spike100Pop contains 1 instance(s) of component: spike100 of type: spikeArray"
        )

        h(" {n_spike100Pop = 1} ")
        """
        Population spike100Pop contains instances of Component(id=spike100 type=spikeArray)
        whose dynamics will be implemented as a mechanism (spike100) in a mod file
        """
        h(" create spike100Pop[1]")
        h(" objectvar m_spike100_spike100Pop[1] ")

        for i in range(int(h.n_spike100Pop)):
            h.spike100Pop[i].L = 10.0
            h.spike100Pop[i](0.5).diam = 10.0
            h.spike100Pop[i](0.5).cm = 318.31
            h.spike100Pop[i].push()
            h(
                " spike100Pop[%i]  { m_spike100_spike100Pop[%i] = new spike100(0.5) } "
                % (i, i)
            )

            h.pop_section()

            self.next_global_id += 1

        # ######################   Population: iafPop
        print(
            "Population iafPop contains 7 instance(s) of component: iaf1 of type: iafCell"
        )

        h(" {n_iafPop = 7} ")
        """
        Population iafPop contains instances of Component(id=iaf1 type=iafCell)
        whose dynamics will be implemented as a mechanism (iaf1) in a mod file
        """
        h(" create iafPop[7]")
        h(" objectvar m_iaf1_iafPop[7] ")

        for i in range(int(h.n_iafPop)):
            h.iafPop[i].L = 10.0
            h.iafPop[i](0.5).diam = 10.0
            h.iafPop[i](0.5).cm = 3.183098861837907
            h.iafPop[i].push()
            h(" iafPop[%i]  { m_iaf1_iafPop[%i] = new iaf1(0.5) } " % (i, i))

            h.m_iaf1_iafPop[i].leakConductance = 5.0e-4
            h.m_iaf1_iafPop[i].leakReversal = -60.0
            h.m_iaf1_iafPop[i].thresh = -35.0
            h.m_iaf1_iafPop[i].reset = -65.0
            h.m_iaf1_iafPop[i].C = 1.0e-5
            h.pop_section()

            self.next_global_id += 1

        # ######################   Population: if1Pop
        print(
            "Population if1Pop contains 7 instance(s) of component: if1 of type: intFire1Cell"
        )

        h(" {n_if1Pop = 7} ")
        """
        Population if1Pop contains instances of Component(id=if1 type=intFire1Cell)
        whose dynamics will be implemented as a mechanism (if1) in a mod file
        """
        h(" create if1Pop[7]")
        h(" objectvar m_if1_if1Pop[7] ")

        for i in range(int(h.n_if1Pop)):
            h.if1Pop[i].L = 10.0
            h.if1Pop[i](0.5).diam = 10.0
            h.if1Pop[i](0.5).cm = 318.31
            h.if1Pop[i].push()
            h(" if1Pop[%i]  { m_if1_if1Pop[%i] = new if1(0.5) } " % (i, i))

            h.m_if1_if1Pop[i].tau = 24.0
            h.m_if1_if1Pop[i].C = 1.0e-5
            h.m_if1_if1Pop[i].thresh = 1.0
            h.m_if1_if1Pop[i].reset = 0.0
            h.m_if1_if1Pop[i].MVOLT = 1.0
            h.m_if1_if1Pop[i].PAMPERE = 0.001
            h.pop_section()

            self.next_global_id += 1

        # ######################   Projection: projection1
        print(
            "Adding projection: projection1, from spike100Pop to iafPop with synapse syn1, 1 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 0, seg 0 (0.5) [0.5 on iafPop[0]], weight: 1.0, delay 0.0
        h("objectvar syn_iafPop_0_0_syn1_0")
        h("iafPop[0] syn_iafPop_0_0_syn1_0 = new syn1(0.5)")
        h("objectvar nc_syn_iafPop_0_0_syn1_0_0")
        h(
            "spike100Pop[0] nc_syn_iafPop_0_0_syn1_0_0 = new NetCon(m_spike100_spike100Pop[0], syn_iafPop_0_0_syn1_0, 0.0, 0.0, 1.0)"
        )

        # ######################   Projection: projection2
        print(
            "Adding projection: projection2, from spike100Pop to iafPop with synapse syn1, 2 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 1, seg 0 (0.5) [0.5 on iafPop[1]], weight: 0.5, delay 10.0
        h("objectvar syn_iafPop_1_0_syn1_0")
        h("iafPop[1] syn_iafPop_1_0_syn1_0 = new syn1(0.5)")
        h("objectvar nc_syn_iafPop_1_0_syn1_0_0")
        h(
            "spike100Pop[0] nc_syn_iafPop_1_0_syn1_0_0 = new NetCon(m_spike100_spike100Pop[0], syn_iafPop_1_0_syn1_0, 0.0, 10.0, 0.5)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 2, seg 0 (0.5) [0.5 on iafPop[2]], weight: 0.25, delay 15.0
        h("objectvar syn_iafPop_2_0_syn1_0")
        h("iafPop[2] syn_iafPop_2_0_syn1_0 = new syn1(0.5)")
        h("objectvar nc_syn_iafPop_2_0_syn1_0_1")
        h(
            "spike100Pop[0] nc_syn_iafPop_2_0_syn1_0_1 = new NetCon(m_spike100_spike100Pop[0], syn_iafPop_2_0_syn1_0, 0.0, 15.0, 0.25)"
        )

        # ######################   Projection: projection3
        print(
            "Adding projection: projection3, from spikeArrayPop to iafPop with synapse syn2, 1 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spikeArrayPop[0]] -> cell 3, seg 0 (0.5) [0.5 on iafPop[3]], weight: 1.0, delay 0.0
        h("objectvar syn_iafPop_3_0_syn2_0")
        h("iafPop[3] syn_iafPop_3_0_syn2_0 = new syn2(0.5)")
        h("objectvar nc_syn_iafPop_3_0_syn2_0_0")
        h(
            "spikeArrayPop[0] nc_syn_iafPop_3_0_syn2_0_0 = new NetCon(m_spikes2_spikeArrayPop[0], syn_iafPop_3_0_syn2_0, 0.0, 0.0, 1.0)"
        )

        # ######################   Projection: projection4
        print(
            "Adding projection: projection4, from spikeArrayPop to iafPop with synapse syn2, 2 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spikeArrayPop[0]] -> cell 4, seg 0 (0.5) [0.5 on iafPop[4]], weight: 0.5, delay 10.0
        h("objectvar syn_iafPop_4_0_syn2_0")
        h("iafPop[4] syn_iafPop_4_0_syn2_0 = new syn2(0.5)")
        h("objectvar nc_syn_iafPop_4_0_syn2_0_0")
        h(
            "spikeArrayPop[0] nc_syn_iafPop_4_0_syn2_0_0 = new NetCon(m_spikes2_spikeArrayPop[0], syn_iafPop_4_0_syn2_0, 0.0, 10.0, 0.5)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spikeArrayPop[0]] -> cell 5, seg 0 (0.5) [0.5 on iafPop[5]], weight: 0.25, delay 15.0
        h("objectvar syn_iafPop_5_0_syn2_0")
        h("iafPop[5] syn_iafPop_5_0_syn2_0 = new syn2(0.5)")
        h("objectvar nc_syn_iafPop_5_0_syn2_0_1")
        h(
            "spikeArrayPop[0] nc_syn_iafPop_5_0_syn2_0_1 = new NetCon(m_spikes2_spikeArrayPop[0], syn_iafPop_5_0_syn2_0, 0.0, 15.0, 0.25)"
        )

        # ######################   Projection: projection1a
        print(
            "Adding projection: projection1a, from spike100Pop to if1Pop with synapse syn0, 3 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 0, seg 0 (0.5) [0.5 on if1Pop[0]], weight: 0.1, delay 0.0
        h("objectvar syn_if1Pop_0_0_syn0_0")
        h("if1Pop[0] syn_if1Pop_0_0_syn0_0 = new syn0(0.5)")
        h("objectvar nc_syn_if1Pop_0_0_syn0_0_0")
        h(
            "spike100Pop[0] nc_syn_if1Pop_0_0_syn0_0_0 = new NetCon(m_spike100_spike100Pop[0], syn_if1Pop_0_0_syn0_0, 0.0, 0.0, 0.1)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 1, seg 0 (0.5) [0.5 on if1Pop[1]], weight: 0.05, delay 10.0
        h("objectvar syn_if1Pop_1_0_syn0_0")
        h("if1Pop[1] syn_if1Pop_1_0_syn0_0 = new syn0(0.5)")
        h("objectvar nc_syn_if1Pop_1_0_syn0_0_1")
        h(
            "spike100Pop[0] nc_syn_if1Pop_1_0_syn0_0_1 = new NetCon(m_spike100_spike100Pop[0], syn_if1Pop_1_0_syn0_0, 0.0, 10.0, 0.05)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spike100Pop[0]] -> cell 2, seg 0 (0.5) [0.5 on if1Pop[2]], weight: 0.2, delay 15.0
        h("objectvar syn_if1Pop_2_0_syn0_0")
        h("if1Pop[2] syn_if1Pop_2_0_syn0_0 = new syn0(0.5)")
        h("objectvar nc_syn_if1Pop_2_0_syn0_0_2")
        h(
            "spike100Pop[0] nc_syn_if1Pop_2_0_syn0_0_2 = new NetCon(m_spike100_spike100Pop[0], syn_if1Pop_2_0_syn0_0, 0.0, 15.0, 0.2)"
        )

        # ######################   Projection: projection1b
        print(
            "Adding projection: projection1b, from spikersPop to if1Pop with synapse syn0, 1 connection(s)"
        )

        # Connection null: cell 0, seg 0 (0.5) [0.5 on spikersPop[0]] -> cell 3, seg 0 (0.5) [0.5 on if1Pop[3]], weight: 0.4, delay 0.0
        h("objectvar syn_if1Pop_3_0_syn0_0")
        h("if1Pop[3] syn_if1Pop_3_0_syn0_0 = new syn0(0.5)")
        h("objectvar nc_syn_if1Pop_3_0_syn0_0_0")
        h(
            "spikersPop[0] nc_syn_if1Pop_3_0_syn0_0_0 = new NetCon(m_spiker_spikersPop[0], syn_if1Pop_3_0_syn0_0, 0.0, 0.0, 0.4)"
        )

        trec = h.Vector()
        trec.record(h._ref_t)

        h.tstop = tstop

        h.dt = dt

        h.steps_per_ms = 1 / h.dt

        # ######################   Display: self.display_d1
        self.display_d1 = h.Graph(0)
        self.display_d1.size(0, h.tstop, -80.0, 50.0)
        self.display_d1.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d1)
        # Line, plotting: spikersPop[0]/tsince
        self.display_d1.addexpr(
            "m_spiker_spikersPop[0].tsince",
            "m_spiker_spikersPop[0].tsince",
            1,
            1,
            0.8,
            0.9,
            2,
        )
        # Line, plotting: spike100Pop[0]/tsince
        self.display_d1.addexpr(
            "m_spike100_spike100Pop[0].tsince",
            "m_spike100_spike100Pop[0].tsince",
            2,
            1,
            0.8,
            0.9,
            2,
        )
        # Line, plotting: spikeArrayPop[0]/tsince
        self.display_d1.addexpr(
            "m_spikes2_spikeArrayPop[0].tsince",
            "m_spikes2_spikeArrayPop[0].tsince",
            3,
            1,
            0.8,
            0.9,
            2,
        )

        # ######################   Display: self.display_d2
        self.display_d2 = h.Graph(0)
        self.display_d2.size(0, h.tstop, -80.0, 50.0)
        self.display_d2.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d2)
        # Line, plotting: iafPop[0]/v
        self.display_d2.addexpr(
            "iafPop[0].v(0.5)", "iafPop[0].v(0.5)", 1, 1, 0.8, 0.9, 2
        )
        # Line, plotting: iafPop[1]/v
        self.display_d2.addexpr(
            "iafPop[1].v(0.5)", "iafPop[1].v(0.5)", 2, 1, 0.8, 0.9, 2
        )
        # Line, plotting: iafPop[2]/v
        self.display_d2.addexpr(
            "iafPop[2].v(0.5)", "iafPop[2].v(0.5)", 3, 1, 0.8, 0.9, 2
        )

        # ######################   Display: self.display_d3
        self.display_d3 = h.Graph(0)
        self.display_d3.size(0, h.tstop, -80.0, 50.0)
        self.display_d3.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d3)
        # Line, plotting: iafPop[3]/v
        self.display_d3.addexpr(
            "iafPop[3].v(0.5)", "iafPop[3].v(0.5)", 1, 1, 0.8, 0.9, 2
        )
        # Line, plotting: iafPop[4]/v
        self.display_d3.addexpr(
            "iafPop[4].v(0.5)", "iafPop[4].v(0.5)", 2, 1, 0.8, 0.9, 2
        )
        # Line, plotting: iafPop[5]/v
        self.display_d3.addexpr(
            "iafPop[5].v(0.5)", "iafPop[5].v(0.5)", 3, 1, 0.8, 0.9, 2
        )

        # ######################   Display: self.display_d20
        self.display_d20 = h.Graph(0)
        self.display_d20.size(0, h.tstop, -80.0, 50.0)
        self.display_d20.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d20)
        # Line, plotting: if1Pop[0]/v
        self.display_d20.addexpr(
            "if1Pop[0].v(0.5)", "if1Pop[0].v(0.5)", 1, 1, 0.8, 0.9, 2
        )
        # Line, plotting: if1Pop[1]/v
        self.display_d20.addexpr(
            "if1Pop[1].v(0.5)", "if1Pop[1].v(0.5)", 2, 1, 0.8, 0.9, 2
        )
        # Line, plotting: if1Pop[2]/v
        self.display_d20.addexpr(
            "if1Pop[2].v(0.5)", "if1Pop[2].v(0.5)", 3, 1, 0.8, 0.9, 2
        )

        # ######################   Display: self.display_d20b
        self.display_d20b = h.Graph(0)
        self.display_d20b.size(0, h.tstop, -80.0, 50.0)
        self.display_d20b.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d20b)
        # Line, plotting: if1Pop[3]/v
        self.display_d20b.addexpr(
            "if1Pop[3].v(0.5)", "if1Pop[3].v(0.5)", 1, 1, 0.8, 0.9, 2
        )

        # ######################   Display: self.display_d20a
        self.display_d20a = h.Graph(0)
        self.display_d20a.size(0, h.tstop, -80.0, 50.0)
        self.display_d20a.view(0, -80.0, h.tstop, 130.0, 80, 330, 330, 250)
        h.graphList[0].append(self.display_d20a)
        # Line, plotting: if1Pop[0]/syn0/i
        self.display_d20a.addexpr(
            "m_if1_if1Pop[0].syn0_i", "m_if1_if1Pop[0].syn0_i", 1, 1, 0.8, 0.9, 2
        )
        # Line, plotting: if1Pop[1]/syn0/i
        self.display_d20a.addexpr(
            "m_if1_if1Pop[1].syn0_i", "m_if1_if1Pop[1].syn0_i", 2, 1, 0.8, 0.9, 2
        )
        # Line, plotting: if1Pop[2]/syn0/i
        self.display_d20a.addexpr(
            "m_if1_if1Pop[2].syn0_i", "m_if1_if1Pop[2].syn0_i", 3, 1, 0.8, 0.9, 2
        )

        # ######################   File to save: ex0.dat (of0)
        # Column: iafPop[0]/v
        h(" objectvar v_0_of0 ")
        h(" { v_0_of0 = new Vector() } ")
        h(" { v_0_of0.record(&iafPop[0].v(0.5)) } ")
        h.v_0_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[1]/v
        h(" objectvar v_1_of0 ")
        h(" { v_1_of0 = new Vector() } ")
        h(" { v_1_of0.record(&iafPop[1].v(0.5)) } ")
        h.v_1_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[2]/v
        h(" objectvar v_2_of0 ")
        h(" { v_2_of0 = new Vector() } ")
        h(" { v_2_of0.record(&iafPop[2].v(0.5)) } ")
        h.v_2_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[3]/v
        h(" objectvar v_3_of0 ")
        h(" { v_3_of0 = new Vector() } ")
        h(" { v_3_of0.record(&iafPop[3].v(0.5)) } ")
        h.v_3_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[4]/v
        h(" objectvar v_4_of0 ")
        h(" { v_4_of0 = new Vector() } ")
        h(" { v_4_of0.record(&iafPop[4].v(0.5)) } ")
        h.v_4_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[5]/v
        h(" objectvar v_5_of0 ")
        h(" { v_5_of0 = new Vector() } ")
        h(" { v_5_of0.record(&iafPop[5].v(0.5)) } ")
        h.v_5_of0.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: iafPop[6]/v
        h(" objectvar v_6_of0 ")
        h(" { v_6_of0 = new Vector() } ")
        h(" { v_6_of0.record(&iafPop[6].v(0.5)) } ")
        h.v_6_of0.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: time.dat (time)
        # Column: time
        h(" objectvar v_time ")
        h(" { v_time = new Vector() } ")
        h(" { v_time.record(&t) } ")
        h.v_time.resize((h.tstop * h.steps_per_ms) + 1)

        self.initialized = False

        self.sim_end = -1  # will be overwritten

        h.nrncontrolmenu()

    def run(self):

        self.initialized = True
        sim_start = time.time()
        print(
            "Running a simulation of %sms (dt = %sms; seed=%s)"
            % (h.tstop, h.dt, self.seed)
        )

        h.run()

        self.sim_end = time.time()
        self.sim_time = self.sim_end - sim_start
        print(
            "Finished NEURON simulation in %f seconds (%f mins)..."
            % (self.sim_time, self.sim_time / 60.0)
        )

        self.save_results()

    def advance(self):

        if not self.initialized:
            h.finitialize()
            self.initialized = True

        h.fadvance()

    ###############################################################################
    # Hash function to use in generation of random value
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _id32(self, obj):
        return int(
            hashlib.md5(obj).hexdigest()[0:8], 16
        )  # convert 8 first chars of md5 hash in base 16 to int

    ###############################################################################
    # Initialize the stim randomizer
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _init_stim_randomizer(self, rand, stimType, gid, seed):
        # print("INIT STIM  %s; %s; %s; %s"%(rand, stimType, gid, seed))
        rand.Random123(self._id32(stimType), gid, seed)

    def save_results(self):

        print("Saving results at t=%s..." % h.t)

        if self.sim_end < 0:
            self.sim_end = time.time()

        self.display_d1.exec_menu("View = plot")
        self.display_d2.exec_menu("View = plot")
        self.display_d3.exec_menu("View = plot")
        self.display_d20.exec_menu("View = plot")
        self.display_d20b.exec_menu("View = plot")
        self.display_d20a.exec_menu("View = plot")

        # ######################   File to save: time.dat (time)
        py_v_time = [
            t / 1000 for t in h.v_time.to_python()
        ]  # Convert to Python list for speed...

        f_time_f2 = open("time.dat", "w")
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_time_f2.write("%f" % py_v_time[i])  # Save in SI units...
        f_time_f2.close()
        print("Saved data to: time.dat")

        # ######################   File to save: ex0.dat (of0)
        py_v_0_of0 = [
            float(x / 1000.0) for x in h.v_0_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_1_of0 = [
            float(x / 1000.0) for x in h.v_1_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_2_of0 = [
            float(x / 1000.0) for x in h.v_2_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_3_of0 = [
            float(x / 1000.0) for x in h.v_3_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_4_of0 = [
            float(x / 1000.0) for x in h.v_4_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_5_of0 = [
            float(x / 1000.0) for x in h.v_5_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_6_of0 = [
            float(x / 1000.0) for x in h.v_6_of0.to_python()
        ]  # Convert to Python list for speed, variable has dim: voltage

        f_of0_f2 = open("ex0.dat", "w")
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_of0_f2.write(
                "%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t\n"
                % (
                    py_v_time[i],
                    py_v_0_of0[i],
                    py_v_1_of0[i],
                    py_v_2_of0[i],
                    py_v_3_of0[i],
                    py_v_4_of0[i],
                    py_v_5_of0[i],
                    py_v_6_of0[i],
                )
            )
        f_of0_f2.close()
        print("Saved data to: ex0.dat")

        save_end = time.time()
        save_time = save_end - self.sim_end
        print("Finished saving results in %f seconds" % (save_time))

        print("Done")


if __name__ == "__main__":

    ns = NeuronSimulation(tstop=300, dt=0.005, seed=123456789)

    ns.run()



import os
import sys
from neuromllite.NetworkGenerator import check_to_generate_or_run

os.chdir("..")
sys.path.append(".")

sys.argv.append("-jnml")

from Example7 import generate

sim, net = generate()

check_to_generate_or_run(sys.argv, sim)
